@@ -1,6 +1,4 @@
-
-/*
- * nmh.h -- system configuration header file
+/* nmh.h -- system configuration header file
  */
 
 #ifdef HAVE_CONFIG_H
@@ -15,11 +13,19 @@
 # include <sbr/ctype-checked.h>
 #endif
 #include <assert.h>
+
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#else
+# define bool int
+# define true 1
+# define false 0
+#endif
+
 #include <sys/stat.h>
 #include <sys/wait.h>
 
 # include <dirent.h>
-#define NLENGTH(dirent) strlen((dirent)->d_name)
 
 #include <stdlib.h>
 #include <stdarg.h>
@@ -68,11 +74,13 @@
 # endif
 #endif
 
+#ifndef HAVE_GETLINE
+ssize_t getline(char **lineptr, size_t *n, FILE *stream);
+#endif
+
 /*
  * Defaults for programs if they aren't configured in a user's profile
  */
 
 #define DEFAULT_PAGER "more"
 #define DEFAULT_EDITOR "vi"
-
-#include <signal.h>
