@@ -1,8 +1,4 @@
-
-/*
- * context_foil.c -- foil search of profile and context
- *
- * $Id: context_foil.c,v 1.4 2006/01/02 03:17:42 bress Exp $
+/* context_foil.c -- foil search of profile and context
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -20,13 +16,13 @@
 int
 context_foil (char *path)
 {
-    register struct node *np;
+    struct node *np;
 
     /* In fact, nobody examines defpath in code paths where
      * it's been set by us -- the uses in the source tree are:
      *  1 sbr/context_read.c uses it only after setting it itself
      *  2 uip/install_mh.c uses it only after setting it itself
-     *  3 uip/mshcmds.c and uip/mark.c print it if given the -debug switch
+     *  3 uip/mark.c prints it if given the -debug switch
      * A worthwhile piece of code cleanup would be to make 1 and
      * 2 use a local variable and just delete 3.
      *
@@ -46,15 +42,14 @@ context_foil (char *path)
      * If path is given, create a minimal profile/context list
      */
     if (path) {
-	m_defs = (struct node *) mh_xmalloc (sizeof(*np));
-
-	np = m_defs;
+	NEW(np);
+	m_defs = np;
 	if (!(np->n_name = strdup ("Path"))) {
-	    advise (NULL, "strdup failed");
+	    inform("strdup failed");
 	    return -1;
 	}
 	if (!(np->n_field = strdup (path))) {
-	    advise (NULL, "strdup failed");
+	    inform("strdup failed");
 	    return -1;
 	}
 	np->n_context = 0;
@@ -62,11 +57,10 @@ context_foil (char *path)
 
 	if (mypath == NULL && (mypath = getenv ("HOME")) != NULL)
 	    if (!(mypath = strdup (mypath))) {
-		advise (NULL, "strdup failed");
+		inform("strdup failed");
 		return -1;
 	    }
     }
 
     return 0;
 }
-
