@@ -1,6 +1,4 @@
-
-/*
- * context_replace.c -- add/replace an entry in the context/profile list
+/* context_replace.c -- add/replace an entry in the context/profile list
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -14,15 +12,14 @@
 void
 context_replace (char *key, char *value)
 {
-    register struct node *np;
+    struct node *np;
 
     /*
-     * If list is emtpy, allocate head of profile/context list.
+     * If list is empty, allocate head of profile/context list.
      */
     if (!m_defs) {
-	m_defs = (struct node *) mh_xmalloc (sizeof(*np));
-
-	np = m_defs;
+	NEW(np);
+        m_defs = np;
 	np->n_name = getcpy (key);
 	np->n_field = getcpy (value);
 	np->n_context = 1;
@@ -36,13 +33,12 @@ context_replace (char *key, char *value)
      * this key, and replace its value if found.
      */
     for (np = m_defs;; np = np->n_next) {
-	if (!strcasecmp (np->n_name ? np->n_name : "", key ? key : "")) {
+	if (!strcasecmp (FENDNULL(np->n_name), FENDNULL(key))) {
 	    if (strcmp (value, np->n_field)) {
 		if (!np->n_context)
-		    admonish (NULL, "bug: context_replace(key=\"%s\",value=\"%s\")", key, value);
-		if (np->n_field)
-		    free (np->n_field);
-		np->n_field = getcpy (value);
+		    inform("bug: context_replace(key=\"%s\",value=\"%s\"), continuing...", key, value);
+                mh_xfree(np->n_field);
+		np->n_field = mh_xstrdup(value);
 		ctxflags |= CTXMOD;
 	    }
 	    return;
@@ -54,8 +50,7 @@ context_replace (char *key, char *value)
     /*
      * Else add this new entry at the end
      */
-    np->n_next = (struct node *) mh_xmalloc (sizeof(*np));
-
+    NEW(np->n_next);
     np = np->n_next;
     np->n_name = getcpy (key);
     np->n_field = getcpy (value);
