@@ -1,6 +1,4 @@
-
-/*
- * cpydgst.c -- copy from one fd to another in encapsulating mode
+/* cpydgst.c -- copy from one fd to another in encapsulating mode
  *           -- (do dashstuffing of input data).
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
@@ -35,9 +33,9 @@
 void
 cpydgst (int in, int out, char *ifile, char *ofile)
 {
-    register int i, j, state;
-    register char *cp, *ep;
-    register char *bp, *dp;
+    int i, j, state;
+    char *cp, *ep;
+    char *bp, *dp;
     char buffer[BUFSIZ], outbuf[BUFSIZ];
 
     dp = (bp = outbuf) + sizeof outbuf;
@@ -51,8 +49,8 @@ cpydgst (int in, int out, char *ifile, char *ofile)
 			output ('-');
 			output (' ');
 		    }
-		    state = S2;	/* fall */
-
+		    state = S2;
+		    /* FALLTHRU */
 		case S2: 
 		    output (*cp);
 		    if (*cp == '\n')
