@@ -1,6 +1,4 @@
-
-/*
- * dtime.c -- time/date routines
+/* dtime.c -- time/date routines
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -9,6 +7,7 @@
 
 #include <h/mh.h>   /* for snprintf() */
 #include <h/nmh.h>
+#include <h/utils.h>
 #include <h/tws.h>
 #include <time.h>
 
@@ -41,12 +40,6 @@ char *tw_ldotw[] = {
     "Saturday",  NULL
 };
 
-struct zone {
-    char *std;
-    char *dst;
-    int shift;
-};
-
 static int dmsize[] = {
     31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
 };
@@ -108,13 +101,19 @@ dlocaltime (time_t *clock)
     if (tm->tm_isdst)			/* if DST is in effect */
 	tw.tw_zone -= 60;		/* reset to normal offset */
 #else
-    tzset();
+    {
+        static bool deja_vu;
+
+        if (!deja_vu) {
+            deja_vu = true;
+            tzset();
+        }
+    }
     tw.tw_zone = -(timezone / 60);
 #endif
 
     tw.tw_flags &= ~TW_SDAY;
     tw.tw_flags |= TW_SEXP;
-    tw.tw_flags &= ~TW_SZONE;
     tw.tw_flags |= TW_SZEXP;
 
     tw.tw_clock = *clock;
@@ -162,7 +161,6 @@ dgmtime (time_t *clock)
 
     tw.tw_flags &= ~TW_SDAY;
     tw.tw_flags |= TW_SEXP;
-    tw.tw_flags &= ~TW_SZONE;
     tw.tw_flags |= TW_SZEXP;
 
     tw.tw_clock = *clock;
@@ -186,10 +184,10 @@ dctime (struct tws *tw)
     if (!tw)
 	return NULL;
 
-    snprintf (buffer, sizeof(buffer), "%.3s %.3s %02d %02d:%02d:%02d %.4d\n",
+    snprintf (buffer, sizeof(buffer), "%.3s %.3s %02d %02d:%02d:%02d %s\n",
 	    tw_dotw[tw->tw_wday], tw_moty[tw->tw_mon], tw->tw_mday,
 	    tw->tw_hour, tw->tw_min, tw->tw_sec,
-	    tw->tw_year < 100 ? tw->tw_year + 1900 : tw->tw_year);
+	    m_strn(tw->tw_year < 100 ? tw->tw_year + 1900 : tw->tw_year, 4));
 
     return buffer;
 }
@@ -201,8 +199,8 @@ dctime (struct tws *tw)
  *	Mon, 16 Jun 1992 15:30:48 -700 (or)
  *	Mon, 16 Jun 1992 15:30:48 EDT
  *
- * for the current time, as specified by rfc822.
- * The first form is required by rfc1123.
+ * for the current time, as specified by RFC 822.
+ * The first form is required by RFC 1123.
  */
 
 char *
@@ -222,8 +220,8 @@ dtimenow (int alpha_timezone)
  *	Mon, 16 Jun 1992 15:30:48 -700  (or)
  *	Mon, 16 Jun 1992 15:30:48 EDT
  *
- * as specified by rfc822.  The first form is required
- * by rfc1123 for outgoing messages.
+ * as specified by RFC 822.  The first form is required
+ * by RFC 1123 for outgoing messages.
  */
 
 char *
@@ -232,9 +230,9 @@ dtime (time_t *clock, int alpha_timezone)
     if (alpha_timezone)
 	/* use alpha-numeric timezones */
 	return dasctime (dlocaltime (clock), TW_NULL);
-    else
-	/* use numeric timezones */
-	return dasctime (dlocaltime (clock), TW_ZONE);
+
+    /* use numeric timezones */
+    return dasctime (dlocaltime (clock), TW_ZONE);
 }
 
 
@@ -244,7 +242,7 @@ dtime (time_t *clock, int alpha_timezone)
  *
  *	Mon, 16 Jun 1992 15:30:48 -0700
  *
- * as specified by rfc822 and rfc1123.
+ * as specified by RFC 822 and RFC 1123.
  */
 
 char *
@@ -252,28 +250,29 @@ dasctime (struct tws *tw, int flags)
 {
     char buffer[80];
     static char result[80];
+    int twf;
 
     if (!tw)
 	return NULL;
 
     /* Display timezone if known */
-    if ((tw->tw_flags & TW_SZONE) == TW_SZNIL)
-	result[0] = '\0';
-    else
+    if (tw->tw_flags & TW_SZEXP)
 	snprintf(result, sizeof(result), " %s", dtimezone(tw->tw_zone, tw->tw_flags | flags));
+    else
+	result[0] = '\0';
 
     snprintf(buffer, sizeof(buffer), "%02d %s %0*d %02d:%02d:%02d%s",
 	    tw->tw_mday, tw_moty[tw->tw_mon],
 	    tw->tw_year < 100 ? 2 : 4, tw->tw_year,
 	    tw->tw_hour, tw->tw_min, tw->tw_sec, result);
 
-    if ((tw->tw_flags & TW_SDAY) == TW_SEXP)
-	snprintf (result, sizeof(result), "%s, %s", tw_dotw[tw->tw_wday], buffer);
-    else
-	if ((tw->tw_flags & TW_SDAY) == TW_SNIL)
-	    strncpy (result, buffer, sizeof(result));
-	else
-	    snprintf (result, sizeof(result), "%s (%s)", buffer, tw_dotw[tw->tw_wday]);
+    if ((twf = tw->tw_flags & TW_SDAY)) {
+        if (twf == TW_SEXP)
+            snprintf(result, sizeof(result), "%s, %s", tw_dotw[tw->tw_wday], buffer);
+        else
+	    snprintf(result, sizeof(result), "%s (%s)", buffer, tw_dotw[tw->tw_wday]);
+    } else
+        strncpy(result, buffer, sizeof(result));
 
     return result;
 }
@@ -286,26 +285,23 @@ dasctime (struct tws *tw, int flags)
  * return the string representation of the numeric offset.
  */
 
-char *
-dtimezone (int offset, int flags)
+char *dtimezone(int offset, int flags)
 {
-    int hours, mins;
-    static char buffer[10];
+    static char buffer[sizeof "+3579139459"]; /* 2,147,483,648 / 60 = 35,791,394 */
+    bool pos;
+    unsigned os, hours, mins;
 
-    if (offset < 0) {
-	mins = -((-offset) % 60);
-	hours = -((-offset) / 60);
-    } else {
-	mins = offset % 60;
-	hours = offset / 60;
-    }
+    pos = offset >= 0;
+    os = pos ? offset : ~offset + 1; /* abs(3) undefined on INT_MIN. */
+    hours = os / 60;
+    mins = os % 60;
+
+    if (flags & TW_DST) /* Shift towards +infinity. */
+        hours += pos ? 1 : -1;
+
+    snprintf(buffer, sizeof(buffer), "%c%02u%02u",
+        pos ? '+' : '-', hours, mins);
 
-#ifdef ADJUST_NUMERIC_ONLY_TZ_OFFSETS_WRT_DST
-    if (flags & TW_DST)
-	hours += 1;
-#endif /* ADJUST_NUMERIC_ONLY_TZ_OFFSETS_WRT_DST */
-    snprintf (buffer, sizeof(buffer), "%s%02d%02d",
-		offset < 0 ? "-" : "+", abs (hours), abs (mins));
     return buffer;
 }
 
@@ -390,18 +386,8 @@ set_dotw (struct tws *tw)
     if (tw->tw_wday < 0)
 	tw->tw_wday += 7;
 
-    tw->tw_flags &= ~TW_SDAY, tw->tw_flags |= TW_SIMP;
-}
-
-
-/*
- * Copy nmh time structure
- */
-
-void
-twscopy (struct tws *tb, struct tws *tw)
-{
-    *tb = *tw;  /* struct copy */
+    tw->tw_flags &= ~TW_SDAY;
+    tw->tw_flags |= TW_SIMP;
 }
 
 
