@@ -1,6 +1,4 @@
-/*
- *
- *	Run a program that hooks into some other system.  The first argument is
+/*	Run a program that hooks into some other system.  The first argument is
  *	name of the hook to use, the second is the full path name of a mail message.
  *	The third argument is also the full path name of a mail message, or a NULL
  *	pointer if it isn't needed.  Look in the context for an error message if
@@ -22,13 +20,13 @@ ext_hook(char *hook_name, char *message_file_name_1, char *message_file_name_2)
 
     static  int	did_message = 0;	/* set if we've already output a message */
 
-    if ((hook = context_find(hook_name)) == (char *)0)
+    if ((hook = context_find(hook_name)) == NULL)
 	return (OK);
 
     switch (pid = fork()) {
     case -1:
 	status = NOTOK;
-	advise(NULL, "external database may be out-of-date.");
+	inform("external database may be out-of-date.");
 	break;
 
     case 0:
@@ -49,8 +47,8 @@ ext_hook(char *hook_name, char *message_file_name_1, char *message_file_name_2)
     if (status != OK) {
 	if (did_message == 0) {
 	    char *msghook;
-	    if ((msghook = context_find("msg-hook")) != (char *)0)
-		advise(NULL, msghook);
+	    if ((msghook = context_find("msg-hook")) != NULL)
+		inform(msghook);
 	    else {
 	    	char errbuf[BUFSIZ];
 		snprintf(errbuf, sizeof(errbuf), "external hook \"%s\"", hook);
