@@ -1,6 +1,4 @@
-
-/*
- * fmt_rfc2047.c -- decode RFC-2047 header format 
+/* fmt_rfc2047.c -- decode RFC-2047 header format 
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -128,7 +126,7 @@ decode_rfc2047 (char *str, char *dst, size_t dstlen)
 	equals_pending = 1;	/* we have a '=' pending */
 
 	/* Check for initial =? */
-	if (*p == '=' && p[1] && p[1] == '?' && p[2]) {
+	if (*p == '=' && p[1] == '?' && p[2]) {
 	    startofmime = p + 2;
 
 	    /* Scan ahead for the next '?' character */
@@ -191,9 +189,9 @@ decode_rfc2047 (char *str, char *dst, size_t dstlen)
 	     */
 	    endofmime = NULL;
 	    for (pp = startofmime; *pp && *(pp+1); pp++) {
-		if (is_lws(*pp)) {
+		if (is_lws(*pp))
 		    break;
-		} else if (*pp == '?' && pp[1] == '=') {
+		if (*pp == '?' && pp[1] == '=') {
 		    endofmime = pp;
 		    break;
 		}
