@@ -1,10 +1,6 @@
-
-/*
- * folder_pack.c -- pack (renumber) the messages in a folder
+/* folder_pack.c -- pack (renumber) the messages in a folder
  *               -- into a contiguous range from 1 to n.
  *
- * $Id: folder_pack.c,v 1.4 2004/11/16 18:14:34 jon Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -40,7 +36,7 @@ folder_pack (struct msgs **mpp, int verbose)
 	if ((mp = folder_realloc (mp, 1, mp->hghmsg)))
 	    *mpp = mp;
 	else {
-	    advise (NULL, "unable to allocate folder storage");
+	    inform("unable to allocate folder storage");
 	    return -1;
 	}
     }
