@@ -1,8 +1,4 @@
-
-/*
- * getfolder.c -- get the current or default folder
- *
- * $Id: getfolder.c,v 1.2 2002/07/02 22:09:14 kenh Exp $
+/* getfolder.c -- get the current or default folder
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -15,7 +11,7 @@
 char *
 getfolder(int wantcurrent)
 {
-    register char *folder;
+    char *folder;
 
     /*
      * If wantcurrent == 1, then try the current folder first
