@@ -1,5 +1,4 @@
-/*
- * m_rand -- provides pseudorandom bytes
+/* m_rand.c -- provides pseudorandom bytes
  *
  * This code is Copyright (c) 2012, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -12,6 +11,7 @@
 #include <time.h>    /* for time() */
 
 #include <config.h>
+#include "m_rand.h"
 
 #if !HAVE_ARC4RANDOM
 static int seeded = 0;
