@@ -1,12 +1,11 @@
-
-/*
- * mf.c -- mail filter subroutines
+/* mf.c -- mail filter subroutines
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
+#include <h/mh.h>
 #include <h/mf.h>
 #include <h/utils.h>
 
@@ -23,40 +22,13 @@ static int route (char *);
 static int my_lex (char *);
 
 
-int
-isfrom(const char *string)
-{
-    return (strncmp (string, "From ", 5) == 0
-	    || strncmp (string, ">From ", 6) == 0);
-}
-
-
-int
-lequal (const char *a, const char *b)
-{
-    for (; *a; a++, b++)
-	if (*b == 0)
-	    return FALSE;
-	else {
-	    char c1 = islower ((unsigned char) *a) ?
-	    				toupper ((unsigned char) *a) : *a;
-	    char c2 = islower ((unsigned char) *b) ?
-	    				toupper ((unsigned char) *b) : *b;
-	    if (c1 != c2)
-		return FALSE;
-	}
-
-    return (*b == 0);
-}
-
-
 static int
 isat (const char *p)
 {
-    return (strncmp (p, " AT ", 4)
-	    && strncmp (p, " At ", 4)
-	    && strncmp (p, " aT ", 4)
-	    && strncmp (p, " at ", 4) ? FALSE : TRUE);
+    return *p == ' ' &&
+        (p[1] == 'a' || p[1] == 'A') &&
+        (p[2] == 't' || p[2] == 'T') &&
+        p[3] == ' ';
 }
 
 
@@ -65,7 +37,7 @@ isat (const char *p)
  * getadrx() implements a partial 822-style address parser.  The parser
  * is neither complete nor correct.  It does however recognize nearly all
  * of the 822 address syntax.  In addition it handles the majority of the
- * 733 syntax as well.  Most problems arise from trying to accomodate both.
+ * 733 syntax as well.  Most problems arise from trying to accommodate both.
  *
  * In terms of 822, the route-specification in 
  *
@@ -165,7 +137,7 @@ static char *ap = NULL;
 static char *pers = NULL;
 static char *mbox = NULL;
 static char *host = NULL;
-static char *path = NULL;
+static char *routepath = NULL;
 static char *grp = NULL;
 static char *note = NULL;
 static char err[BUFSIZ];
@@ -174,29 +146,24 @@ static char adr[BUFSIZ];
 static struct adrx  adrxs2;
 
 
+/* eai = Email Address Internationalization */
 struct adrx *
-getadrx (const char *addrs)
+getadrx (const char *addrs, int eai)
 {
-    register char *bp;
-    register struct adrx *adrxp = &adrxs2;
+    char *bp;
+    struct adrx *adrxp = &adrxs2;
 
-    if (pers)
-	free (pers);
-    if (mbox)
-	free (mbox);
-    if (host)
-	free (host);
-    if (path)
-	free (path);
-    if (grp)
-	free (grp);
-    if (note)
-	free (note);
-    pers = mbox = host = path = grp = note = NULL;
+    mh_xfree(pers);
+    mh_xfree(mbox);
+    mh_xfree(host);
+    mh_xfree(routepath);
+    mh_xfree(grp);
+    mh_xfree(note);
+    pers = mbox = host = routepath = grp = note = NULL;
     err[0] = 0;
 
     if (dp == NULL) {
-	dp = cp = strdup (addrs ? addrs : "");
+	dp = cp = strdup (FENDNULL(addrs));
 	glevel = 0;
     }
     else
@@ -230,13 +197,15 @@ getadrx (const char *addrs)
 	    break;
 	}
 
-    /*
-     * Reject the address if key fields contain 8bit characters
-     */
+    if (! eai) {
+        /*
+         * Reject the address if key fields contain 8bit characters
+         */
 
-    if (contains8bit(mbox, NULL) || contains8bit(host, NULL) ||
-	contains8bit(path, NULL) || contains8bit(grp, NULL)) {
-    	strcpy(err, "Address contains 8-bit characters");
+        if (contains8bit(mbox, NULL) || contains8bit(host, NULL) ||
+            contains8bit(routepath, NULL) || contains8bit(grp, NULL)) {
+            strcpy(err, "Address contains 8-bit characters");
+        }
     }
 
     if (err[0])
@@ -255,7 +224,7 @@ getadrx (const char *addrs)
     while (isspace ((unsigned char) *ap))
 	ap++;
     if (cp)
-	sprintf (adr, "%.*s", (int)(cp - ap), ap);
+	snprintf(adr, sizeof adr, "%.*s", (int)(cp - ap), ap);
     else
 	strcpy (adr, ap);
     bp = adr + strlen (adr) - 1;
@@ -266,7 +235,7 @@ getadrx (const char *addrs)
     adrxp->pers = pers;
     adrxp->mbox = mbox;
     adrxp->host = host;
-    adrxp->path = path;
+    adrxp->path = routepath;
     adrxp->grp = grp;
     adrxp->ingrp = ingrp;
     adrxp->note = note;
@@ -294,11 +263,10 @@ again: ;
 		strcpy (err, "extraneous semi-colon");
 		return NOTOK;
 	    }
+	    /* FALLTHRU */
 	case LX_COMA: 
-	    if (note) {
-		free (note);
-		note = NULL;
-	    }
+            mh_xfree(note);
+            note = NULL;
 	    goto again;
 
 	case LX_END: 
@@ -314,7 +282,7 @@ again: ;
 	    return OK;		/* why be choosy? */
 
 	default: 
-	    sprintf (err, "illegal address construct (%s)", buffer);
+	    snprintf(err, sizeof err, "illegal address construct (%s)", buffer);
 	    return NOTOK;
     }
 
@@ -333,13 +301,13 @@ again: ;
 			return NOTOK;
 		    if (last_lex == LX_RBRK)
 			return OK;
-		    sprintf (err, "missing right-bracket (%s)", buffer);
+		    snprintf(err, sizeof err, "missing right-bracket (%s)", buffer);
 		    return NOTOK;
 
 		case LX_COLN: 
 	    get_group: ;
 		    if (glevel++ > 0) {
-			sprintf (err, "nested groups not allowed (%s)", pers);
+			snprintf(err, sizeof err, "nested groups not allowed (%s)", pers);
 			return NOTOK;
 		    }
 		    grp = add (": ", pers);
@@ -368,7 +336,7 @@ again: ;
 		    goto more_phrase;
 
 		default: 
-		    sprintf (err, "no mailbox in address, only a phrase (%s%s)",
+		    snprintf(err, sizeof err, "no mailbox in address, only a phrase (%s%s)",
 			    pers, buffer);
 		    return NOTOK;
 	    }
@@ -399,12 +367,13 @@ again: ;
 			strcpy (err, "extraneous semi-colon");
 			return NOTOK;
 		    }
+		    return OK;
 		case LX_COMA: 
 		case LX_END: 
 		    return OK;
 
 		default: 
-		    sprintf (err, "junk after local@domain (%s)", buffer);
+		    snprintf(err, sizeof err, "junk after local@domain (%s)", buffer);
 		    return NOTOK;
 	    }
 
@@ -421,7 +390,7 @@ again: ;
 	    return OK;
 
 	default: 
-	    sprintf (err, "missing mailbox (%s)", buffer);
+	    snprintf(err, sizeof err, "missing mailbox (%s)", buffer);
 	    return NOTOK;
     }
 }
@@ -446,7 +415,7 @@ phrase (char *buffer)
 static int
 route_addr (char *buffer)
 {
-    register char *pp = cp;
+    char *pp = cp;
 
     if (my_lex (buffer) == LX_AT) {
 	if (route (buffer) == NOTOK)
@@ -469,7 +438,7 @@ route_addr (char *buffer)
 	    return OK;
 
 	default: 
-	    sprintf (err, "no at-sign after local-part (%s)", buffer);
+	    snprintf(err, sizeof err, "no at-sign after local-part (%s)", buffer);
 	    return NOTOK;
     }
 }
@@ -488,7 +457,7 @@ local_part (char *buffer)
 		break;
 
 	    default: 
-		sprintf (err, "no mailbox in local-part (%s)", buffer);
+		snprintf(err, sizeof err, "no mailbox in local-part (%s)", buffer);
 		return NOTOK;
 	}
 
@@ -515,7 +484,7 @@ domain (char *buffer)
 		break;
 
 	    default: 
-		sprintf (err, "no sub-domain in domain-part of address (%s)", buffer);
+		snprintf(err, sizeof err, "no sub-domain in domain-part of address (%s)", buffer);
 		return NOTOK;
 	}
 
@@ -540,33 +509,33 @@ domain (char *buffer)
 static int
 route (char *buffer)
 {
-    path = strdup ("@");
+    routepath = mh_xstrdup ("@");
 
     for (;;) {
 	switch (my_lex (buffer)) {
 	    case LX_ATOM: 
 	    case LX_DLIT: 
-		path = add (buffer, path);
+		routepath = add (buffer, routepath);
 		break;
 
 	    default: 
-		sprintf (err, "no sub-domain in domain-part of address (%s)", buffer);
+		snprintf(err, sizeof err, "no sub-domain in domain-part of address (%s)", buffer);
 		return NOTOK;
 	}
 	switch (my_lex (buffer)) {
 	    case LX_COMA: 
-		path = add (buffer, path);
+		routepath = add (buffer, routepath);
 		for (;;) {
 		    switch (my_lex (buffer)) {
 			case LX_COMA: 
 			    continue;
 
 			case LX_AT: 
-			    path = add (buffer, path);
+			    routepath = add (buffer, routepath);
 			    break;
 
 			default: 
-			    sprintf (err, "no at-sign found for next domain in route (%s)",
+			    snprintf(err, sizeof err, "no at-sign found for next domain in route (%s)",
 			             buffer);
 		    }
 		    break;
@@ -575,15 +544,15 @@ route (char *buffer)
 
 	    case LX_AT:		/* XXX */
 	    case LX_DOT: 
-		path = add (buffer, path);
+		routepath = add (buffer, routepath);
 		continue;
 
 	    case LX_COLN: 
-		path = add (buffer, path);
+		routepath = add (buffer, routepath);
 		return OK;
 
 	    default: 
-		sprintf (err, "no colon found to terminate route (%s)", buffer);
+		snprintf(err, sizeof err, "no colon found to terminate route (%s)", buffer);
 		return NOTOK;
 	}
     }
@@ -631,6 +600,7 @@ my_lex (char *buffer)
 		    continue;
 		case '(': 
 		    i++;
+		    /* FALLTHRU */
 		default: 
 		    ADDCHR(c);
 		    continue;
@@ -658,6 +628,7 @@ my_lex (char *buffer)
 			cp = NULL;
 			return (last_lex = LX_ERR);
 		    }
+		    /* FALLTHRU */
 		default: 
 		    ADDCHR(c);
 		    continue;
@@ -681,6 +652,7 @@ my_lex (char *buffer)
 			cp = NULL;
 			return (last_lex = LX_ERR);
 		    }
+		    /* FALLTHRU */
 		default: 
 		    ADDCHR(c);
 		    continue;
@@ -731,7 +703,7 @@ char *
 legal_person (const char *p)
 {
     int i;
-    register const char *cp;
+    const char *cp;
     static char buffer[BUFSIZ];
 
     if (*p == '"')
@@ -739,65 +711,9 @@ legal_person (const char *p)
     for (cp = p; *cp; cp++)
 	for (i = 0; special[i].lx_chr; i++)
 	    if (*cp == special[i].lx_chr) {
-		sprintf (buffer, "\"%s\"", p);
+		snprintf(buffer, sizeof buffer, "\"%s\"", p);
 		return buffer;
 	    }
 
     return (char *) p;
 }
-
-
-int
-mfgets (FILE *in, char **bp)
-{
-    int i;
-    register char *cp, *dp, *ep;
-    static int len = 0;
-    static char *pp = NULL;
-
-    if (pp == NULL)
-	pp = mh_xmalloc ((size_t) (len = BUFSIZ));
-
-    for (ep = (cp = pp) + len - 2;;) {
-	switch (i = getc (in)) {
-	    case EOF: 
-	eol: 	;
-		if (cp != pp) {
-		    *cp = 0;
-		    *bp = pp;
-		    return OK;
-		}
-	eoh:	;
-		*bp = NULL;
-		free (pp);
-		pp = NULL;
-		return DONE;
-
-	    case 0: 
-		continue;
-
-	    case '\n': 
-		if (cp == pp)	/* end of headers, gobble it */
-		    goto eoh;
-		switch (i = getc (in)) {
-		    default: 	/* end of line */
-		    case '\n': 	/* end of headers, save for next call */
-			ungetc (i, in);
-			goto eol;
-
-		    case ' ': 	/* continue headers */
-		    case '\t': 
-			*cp++ = '\n';
-			break;
-		}		/* fall into default case */
-
-	    default: 
-		*cp++ = i;
-		break;
-	}
-	if (cp >= ep) {
-	    dp = mh_xrealloc (pp, (size_t) (len += BUFSIZ));
-	    cp += dp - pp, ep = (pp = cp) + len - 2;
-	}
-    }
-}
