@@ -1,6 +1,4 @@
-
-/*
- * readconfig.c -- base routine to read nmh configuration files
+/* readconfig.c -- base routine to read nmh configuration files
  *              -- such as nmh profile, context file, or mhn.defaults.
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
@@ -27,14 +25,12 @@ static struct procstr procs[] = {
     { "mailproc",      &mailproc },
     { "mhlproc",       &mhlproc },
     { "moreproc",      &moreproc },
-    { "mshproc",       &mshproc },
     { "packproc",      &packproc },
     { "postproc",      &postproc },
     { "rmmproc",       &rmmproc },
     { "sendproc",      &sendproc },
     { "showmimeproc",  &showmimeproc },
     { "showproc",      &showproc },
-    { "vmhproc",       &vmhproc },
     { "whatnowproc",   &whatnowproc },
     { "whomproc",      &whomproc },
     { NULL,            NULL }
@@ -44,17 +40,17 @@ static struct node **opp = NULL;
 
 
 void
-readconfig (struct node **npp, FILE *ib, char *file, int ctx)
+readconfig (struct node **npp, FILE *ib, const char *file, int ctx)
 {
-    register int state;
-    register char *cp;
-    char name[NAMESZ], field[BUFSIZ];
-    register struct node *np;
-    register struct procstr *ps;
+    int state;
+    char *cp;
+    char name[NAMESZ], field[NMH_BUFSIZ];
+    struct node *np;
+    struct procstr *ps;
     m_getfld_state_t gstate = 0;
 
     if (npp == NULL && (npp = opp) == NULL) {
-	admonish (NULL, "bug: readconfig called but pump not primed");
+	inform("bug: readconfig called but pump not primed, continuing...");
 	return;
     }
 
@@ -63,12 +59,12 @@ readconfig (struct node **npp, FILE *ib, char *file, int ctx)
 	switch (state = m_getfld (&gstate, name, field, &fieldsz, ib)) {
 	    case FLD:
 	    case FLDPLUS:
-		np = (struct node *) mh_xmalloc (sizeof(*np));
+		NEW(np);
 		*npp = np;
 		*(npp = &np->n_next) = NULL;
-		np->n_name = getcpy (name);
+		np->n_name = mh_xstrdup(name);
 		if (state == FLDPLUS) {
-		    cp = getcpy (field);
+		    cp = mh_xstrdup(field);
 		    while (state == FLDPLUS) {
 			fieldsz = sizeof field;
 			state = m_getfld (&gstate, name, field, &fieldsz, ib);
@@ -132,7 +128,7 @@ readconfig (struct node **npp, FILE *ib, char *file, int ctx)
 	   mhn.defaults can be overridden without triggering
 	   warnings.
 
-	   Note that that mhn.defaults, $MHN, $MHBUILD, $MHSHOW, and
+	   Note that mhn.defaults, $MHN, $MHBUILD, $MHSHOW, and
 	   $MHSTORE all put their entries into just one list, m_defs,
 	   the same list that the profile uses. */
 
@@ -140,13 +136,13 @@ readconfig (struct node **npp, FILE *ib, char *file, int ctx)
 	for (np = m_defs; np; np = np->n_next) {
 	    /* Yes, this is O(N^2).  The profile should be small enough so
 	       that's not a performance problem. */
-	    if (strlen (np->n_name) > 0	 &&  strcmp ("#", np->n_name)) {
+	    if (*np->n_name && strcmp("#", np->n_name)) {
 		struct node *np2;
 		for (np2 = np->n_next; np2; np2 = np2->n_next) {
 		    if (! strcasecmp (np->n_name, np2->n_name)) {
-			admonish (NULL, "multiple \"%s\" profile components "
-					"in %s, ignoring \"%s\"",
-				  np->n_name, defpath, np2->n_field);
+			inform("multiple \"%s\" profile components in %s, "
+			    "ignoring \"%s\", continuing...",
+			    np->n_name, defpath, np2->n_field);
 		    }
 		}
 	    }
@@ -159,11 +155,12 @@ readconfig (struct node **npp, FILE *ib, char *file, int ctx)
 
 void
 add_profile_entry (const char *key, const char *value) {
-    struct node *newnode = (struct node *) mh_xmalloc (sizeof *newnode);
+    struct node *newnode;
 
     /* This inserts the new node at the beginning of m_defs because
        that doesn't require traversing it or checking to see if it's
        empty. */
+    NEW(newnode);
     newnode->n_name = getcpy (key);
     newnode->n_field = getcpy (value);
     newnode->n_context = 0;
