@@ -1,6 +1,4 @@
-
-/*
- * seq_read.c -- read the .mh_sequence file and
+/* seq_read.c -- read the .mh_sequence file and
  *            -- initialize sequence information
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
@@ -10,6 +8,7 @@
 
 #include <h/mh.h>
 #include <h/utils.h>
+#include "lock_file.h"
 
 /*
  * static prototypes
@@ -62,7 +61,7 @@ seq_public (struct msgs *mp, int lockflag, int *failed_to_lock)
 {
     int state;
     char *cp, seqfile[PATH_MAX];
-    char name[NAMESZ], field[BUFSIZ];
+    char name[NAMESZ], field[NMH_BUFSIZ];
     FILE *fp;
     m_getfld_state_t gstate = 0;
 
@@ -88,23 +87,23 @@ seq_public (struct msgs *mp, int lockflag, int *failed_to_lock)
 	    case FLD: 
 	    case FLDPLUS:
 		if (state == FLDPLUS) {
-		    cp = getcpy (field);
+		    cp = mh_xstrdup(field);
 		    while (state == FLDPLUS) {
 			fieldsz = sizeof field;
 			state = m_getfld (&gstate, name, field, &fieldsz, fp);
 			cp = add (field, cp);
 		    }
-		    seq_init (mp, getcpy (name), trimcpy (cp));
+		    seq_init (mp, mh_xstrdup(name), trimcpy (cp));
 		    free (cp);
 		} else {
-		    seq_init (mp, getcpy (name), trimcpy (field));
+		    seq_init (mp, mh_xstrdup(name), trimcpy (field));
 		}
 		continue;
 
 	    case BODY:
 	    	lkfclosedata (fp, seqfile);
 		adios (NULL, "no blank lines are permitted in %s", seqfile);
-		/* fall */
+		break;
 
 	    case FILEEOF:
 		break;
@@ -119,7 +118,7 @@ seq_public (struct msgs *mp, int lockflag, int *failed_to_lock)
 
     if (lockflag) {
 	mp->seqhandle = fp;
-	mp->seqname = getcpy(seqfile);
+	mp->seqname = mh_xstrdup(seqfile);
     } else {
 	lkfclosedata (fp, seqfile);
     }
@@ -142,7 +141,7 @@ seq_private (struct msgs *mp)
     char *cp;
     struct node *np;
 
-    alen = strlen ("atr-");
+    alen = LEN("atr-");
     plen = strlen (mp->foldpath) + 1;
 
     for (np = m_defs; np; np = np->n_next) {
@@ -150,7 +149,7 @@ seq_private (struct msgs *mp)
 		&& (j = strlen (np->n_name) - plen) > alen
 		&& *(np->n_name + j) == '-'
 		&& strcmp (mp->foldpath, np->n_name + j + 1) == 0) {
-	    cp = getcpy (np->n_name + alen);
+	    cp = mh_xstrdup(np->n_name + alen);
 	    *(cp + j - alen) = '\0';
 	    if ((i = seq_init (mp, cp, getcpy (np->n_field))) != -1)
 		make_seq_private (mp, i);
@@ -186,7 +185,7 @@ seq_init (struct msgs *mp, char *name, char *field)
      * Search for this sequence name to see if we've seen
      * it already.  If we've seen this sequence before,
      * then clear the bit for this sequence from all the
-     * mesages in this folder.
+     * messages in this folder.
      */
     for (i = 0; i < svector_size (mp->msgattrs); i++) {
 	if (!strcmp (svector_at (mp->msgattrs, i), name)) {
