@@ -1,6 +1,4 @@
-
-/*
- * seq_save.c -- 1) synchronize sequences
+/* seq_save.c -- 1) synchronize sequences
  *            -- 2) save public sequences
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
@@ -10,6 +8,8 @@
 
 #include <h/mh.h>
 #include <h/signals.h>
+#include "lock_file.h"
+#include "m_mktemp.h"
 
 
 /*
@@ -91,7 +91,9 @@ priv:
 		    free(mp->seqname);
 		    mp->seqname = NULL;
 		    rewind(fp);
-		    ftruncate(fileno(fp), 0);
+		    if (ftruncate(fileno(fp), 0) < 0) {
+			advise ("sequence file", "ftruncate");
+		    }
 		} else if ((fp = lkfopendata (seqfile, "w", &failed_to_lock))
 			   == NULL
 			&& (m_unlink (seqfile) == -1 ||
