@@ -1,8 +1,4 @@
-
-/*
- * seq_setprev.c -- set the Previous-Sequence
- *
- * $Id: seq_setprev.c,v 1.2 2002/07/02 22:09:14 kenh Exp $
+/* seq_setprev.c -- set the Previous-Sequence
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -10,6 +6,7 @@
  */
 
 #include <h/mh.h>
+#include <h/utils.h>
 
 /*
  * Add all the messages currently SELECTED to
@@ -29,7 +26,7 @@ seq_setprev (struct msgs *mp)
      * and split them.
      */
     if ((cp = context_find (psequence))) {
-	dp = getcpy (cp);
+	dp = mh_xstrdup(cp);
 	if (!(ap = brkstring (dp, " ", "\n")) || !*ap) {
 	    free (dp);
 	    return;
