@@ -1,16 +1,13 @@
-
-/*
- * seq_setunseen.c -- add/delete all messages which have the SELECT_UNSEEN
+/* seq_setunseen.c -- add/delete all messages which have the SELECT_UNSEEN
  *                 -- bit set to/from the Unseen-Sequence
  *
- * $Id: seq_setunseen.c,v 1.2 2002/07/02 22:09:14 kenh Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
 #include <h/mh.h>
+#include <h/utils.h>
 
 /*
  * We scan through the folder and act upon all messages
@@ -31,7 +28,7 @@ seq_setunseen (struct msgs *mp, int seen)
      * and split them.
      */
     if ((cp = context_find (usequence))) {
-	dp = getcpy (cp);
+	dp = mh_xstrdup(cp);
 	if (!(ap = brkstring (dp, " ", "\n")) || !*ap) {
 	    free (dp);
 	    return;
