@@ -1,6 +1,4 @@
-
-/*
- * ssequal.c -- check if a string is a substring of another
+/* ssequal.c -- check if a string is a substring of another
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -23,7 +21,7 @@
  */
 
 int
-ssequal (char *s1, char *s2)
+ssequal (const char *s1, const char *s2)
 {
     if (!s1)
 	s1 = "";
