@@ -1,5 +1,4 @@
-/*
- * unquote.c: Handle quote removal and quoted-pair strings on
+/* unquote.c -- Handle quote removal and quoted-pair strings on
  * RFC 2822-5322 atoms.
  *
  * This code is Copyright (c) 2013, by the authors of nmh.  See the
@@ -8,6 +7,7 @@
  */
 
 #include <h/mh.h>
+#include "unquote.h"
 
 /*
  * Remove quotes (and handle escape strings) from RFC 5322 quoted-strings.
@@ -40,6 +40,4 @@ unquote_string(const char *input, char *output)
     }
 
     output[m] = '\0';
-
-    return;
 }
