@@ -1,8 +1,4 @@
-
-/*
- * vfgets.c -- virtual fgets
- *
- * $Id: vfgets.c,v 1.4 2006/01/02 03:25:18 bress Exp $
+/* vfgets.c -- virtual fgets
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -42,9 +38,8 @@ wrong_guess:
 	    if (*dp == '\n') {
 		*bp = pp;
 		return 0;
-	    } else {
-		cp = ++dp;
 	    }
+            cp = ++dp;
 	} else {
 	    for (fp = dp - 1, toggle = 0; fp >= cp; fp--) {
 		if (*fp != QUOTE)
