@@ -1,6 +1,4 @@
-
-/*
- * dist.c -- re-distribute a message
+/* dist.c -- re-distribute a message
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -9,6 +7,7 @@
 
 #include <h/mh.h>
 #include <h/utils.h>
+#include "../sbr/m_maildir.h"
 #include <fcntl.h>
 
 #define DIST_SWITCHES \
@@ -218,7 +217,7 @@ main (int argc, char **argv)
 	}
     }
 
-    cwd = getcpy (pwd ());
+    cwd = mh_xstrdup(pwd ());
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
@@ -232,7 +231,8 @@ try_it_again:
     if (stat (drft, &st) != NOTOK) {
 	printf ("Draft \"%s\" exists (%ld bytes).", drft, (long) st.st_size);
 	for (i = LISTDSW; i != YESW;) {
-	    if (!(argp = getans ("\nDisposition? ", isdf ? aqrnl : aqrl)))
+	    if (!(argp = read_switch_multiword ("\nDisposition? ",
+						isdf ? aqrnl : aqrl)))
 		done (1);
 	    switch (i = smatch (*argp, isdf ? aqrnl : aqrl)) {
 		case NOSW: 
@@ -250,7 +250,7 @@ try_it_again:
 			i = YESW;
 		    break;
 		default: 
-		    advise (NULL, "say what?");
+		    inform("say what?");
 		    break;
 	    }
 	}
@@ -291,7 +291,7 @@ try_it_again:
 	    adios (NULL, "only one message at a time!");
     }
 
-    msgnam = file ? file : getcpy (m_name (mp->lowsel));
+    msgnam = file ? file : mh_xstrdup(m_name (mp->lowsel));
 
     dat[0] = mp ? mp->lowsel : 0;
     dat[1] = 0;
