@@ -1,6 +1,4 @@
-
-/*
- * mark.c -- add message(s) to sequences in given folder
+/* mark.c -- add message(s) to sequences in given folder
  *        -- delete messages (s) from sequences in given folder
  *        -- list sequences in given folder
  *
@@ -11,6 +9,7 @@
 
 #include <h/mh.h>
 #include <h/utils.h>
+#include "../sbr/m_maildir.h"
 
 #define MARK_SWITCHES \
     X("add", 0, ADDSW) \
@@ -246,7 +245,7 @@ print_debug (struct msgs *mp)
 /*
  * Print debugging info about all the SELECTED
  * messages and the sequences they are in.
- * Due limitattions of snprintb(), only a limited
+ * Due to limitations of snprintb(), only a limited
  * number of sequences will be printed.  See the
  * comments in sbr/seq_bits.c.
  */
@@ -256,12 +255,12 @@ seq_printdebug (struct msgs *mp)
     int msgnum;
     char buf[BUFSIZ];
 
-    printf ("\n");
+    putchar('\n');
     for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
 	if (is_selected (mp, msgnum))
 	    printf ("%*d: %s\n", DMAXFOLDER, msgnum,
 		    snprintb (buf, sizeof buf,
-			      (unsigned) *bvector_bits (msgstat (mp, msgnum)),
+			      (unsigned) bvector_first_bits (msgstat (mp, msgnum)),
 			      seq_bits (mp)));
     }
 }
