@@ -1,6 +1,4 @@
-
-/*
- * mhbuild.c -- expand/translate MIME composition files
+/* mhbuild.c -- expand/translate MIME composition files
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -9,7 +7,6 @@
 
 #include <h/mh.h>
 #include <fcntl.h>
-#include <h/signals.h>
 #include <h/md5.h>
 #include <h/mts.h>
 #include <h/tws.h>
@@ -17,6 +14,10 @@
 #include <h/mhparse.h>
 #include <h/mhcachesbr.h>
 #include <h/utils.h>
+#include "../sbr/m_maildir.h"
+#include "../sbr/m_mktemp.h"
+#include "mhfree.h"
+#include "mhoutsbr.h"
 
 #define MHBUILD_SWITCHES \
     X("auto", 0, AUTOSW) \
@@ -47,7 +48,7 @@
     X("version", 0, VERSIONSW) \
     X("help", 0, HELPSW) \
     X("debug", -5, DEBUGSW) \
-    X("dist", -4, DISTSW) \
+    X("dist", 0, DISTSW) \
 
 #define X(sw, minchars, id) id,
 DEFINE_SWITCH_ENUM(MHBUILD);
@@ -57,9 +58,11 @@ DEFINE_SWITCH_ENUM(MHBUILD);
 DEFINE_SWITCH_ARRAY(MHBUILD, switches);
 #undef X
 
+/* utf-8 is for Email Address Internationalization, using SMTPUTF8. */
 #define MIMEENCODING_SWITCHES \
     X("base64", 0, BASE64SW) \
     X("quoted-printable", 0, QUOTEDPRINTSW) \
+    X("utf-8", 0, UTF8SW) \
 
 #define X(sw, minchars, id) id,
 DEFINE_SWITCH_ENUM(MIMEENCODING);
@@ -69,12 +72,6 @@ DEFINE_SWITCH_ENUM(MIMEENCODING);
 DEFINE_SWITCH_ARRAY(MIMEENCODING, encodingswitches);
 #undef X
 
-/* mhcachesbr.c */
-extern int rcachesw;
-extern int wcachesw;
-extern char *cache_public;
-extern char *cache_private;
-
 int debugsw = 0;
 
 int listsw   = 0;
@@ -92,10 +89,6 @@ static int unlink_outfile = 0;
 
 static void unlink_done (int) NORETURN;
 
-/* mhbuildsbr.c */
-int output_message (CT, char *);
-int output_message_fp (CT, FILE *, char*);
-
 
 int
 main (int argc, char **argv)
@@ -111,8 +104,9 @@ main (int argc, char **argv)
     FILE *fp = NULL;
     FILE *fp_out = NULL;
     int header_encoding = CE_UNKNOWN;
+    size_t n;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], 2)) { return 1; }
 
     done=unlink_done;
 
@@ -167,9 +161,9 @@ main (int argc, char **argv)
 	    do_cache: ;
 		if (!(cp = *argp++) || *cp == '-')
 		    adios (NULL, "missing argument to %s", argp[-2]);
-		switch (*icachesw = smatch (cp, caches)) {
+		switch (*icachesw = smatch (cp, cache_policy)) {
 		case AMBIGSW:
-		    ambigsw (cp, caches);
+		    ambigsw (cp, cache_policy);
 		    done (1);
 		case UNKWNSW:
 		    adios (NULL, "%s unknown", cp);
@@ -244,6 +238,9 @@ main (int argc, char **argv)
 		case QUOTEDPRINTSW:
 		    header_encoding = CE_QUOTED;
 		    break;
+		case UTF8SW:
+		    header_encoding = CE_8BIT;
+		    break;
 		default:
 		    adios (NULL, "Internal error: algorithm %s", cp);
 		}
@@ -334,13 +331,18 @@ main (int argc, char **argv)
 		  get_temp_dir());
 	}
 
-	/* copy standard input to temporary file */
+	/* save a copy of the name for later removal */
 	strncpy (infile, cp, sizeof(infile));
-	while (fgets (buffer, BUFSIZ, stdin))
-	    fputs (buffer, fp);
-	fclose (fp);
 	unlink_infile = 1;
 
+	/* copy standard input to temporary file */
+	while ((n = fread(buffer, 1, sizeof(buffer), stdin)) > 0) {
+	    if (fwrite(buffer, 1, n, fp) != n) {
+		adios (NULL, "error copying to temporary file");
+	    }
+	}
+	fclose (fp);
+
 	/* build the content structures for MIME message */
 	ct = build_mime (infile, autobuild, dist, directives, header_encoding,
 			 maxunencoded, verbosw);
@@ -354,17 +356,17 @@ main (int argc, char **argv)
 	    if (! (fp = fopen(infile, "r"))) {
 		adios(NULL, "Unable to open %s for reading", infile);
 	    }
-	    while (fgets(buffer, BUFSIZ, fp))
-		fputs(buffer, stdout);
+	    while ((n = fread(buffer, 1, sizeof(buffer), fp)) > 0) {
+		if (fwrite(buffer, 1, n, stdout) != n) {
+		    adios(NULL, "error copying %s to stdout", infile);
+		}
+	    }
 	} else {
 	    /* output the message */
 	    output_message_fp (ct, stdout, NULL);
 	    free_content (ct);
 	}
 
-	(void) m_unlink (infile);
-	unlink_infile = 0;
-
 	done (0);
     }
 
@@ -424,7 +426,7 @@ main (int argc, char **argv)
 }
 
 
-static void
+static void NORETURN
 unlink_done (int status)
 {
     /*
