@@ -1,6 +1,4 @@
-
-/*
- * mhpath.c -- print full pathnames of nmh messages and folders
+/* mhpath.c -- print full pathnames of nmh messages and folders
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -9,6 +7,7 @@
 
 #include <h/mh.h>
 #include <h/utils.h>
+#include "../sbr/m_maildir.h"
 
 #define MHPATH_SWITCHES \
     X("version", 0, VERSIONSW) \
@@ -32,7 +31,7 @@ main(int argc, char **argv)
     struct msgs_array msgs = { 0, 0, NULL };
     struct msgs *mp;
 
-    if (nmh_init(argv[0], 1)) { return 1; }
+    if (nmh_init(argv[0], 2)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -77,7 +76,7 @@ main(int argc, char **argv)
 
     /* If no messages are given, print folder pathname */
     if (!msgs.size) {
-	printf ("%s\n", maildir);
+	puts(maildir);
 	done (0);
     }
 
