@@ -1,6 +1,4 @@
-
-/*
- * mhshowsbr.c -- routines to display the contents of MIME messages
+/* mhshowsbr.c -- routines to display the contents of MIME messages
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -11,18 +9,21 @@
 #include <fcntl.h>
 #include <h/signals.h>
 #include <h/md5.h>
-#include <setjmp.h>
 #include <h/mts.h>
 #include <h/tws.h>
 #include <h/mime.h>
 #include <h/mhparse.h>
 #include <h/fmt_scan.h>
 #include <h/utils.h>
+#include "mhshowsbr.h"
+#include "../sbr/m_mktemp.h"
 #ifdef HAVE_ICONV
 #   include <iconv.h>
 #endif /* ! HAVE_ICONV */
 
 extern int debugsw;
+extern int npart;
+extern int ntype;
 
 int nolist   = 0;
 
@@ -32,38 +33,42 @@ char *progsw = NULL;
 int nomore   = 0;
 char *formsw = NULL;
 
+/* for output markerss and headers */
+char *folder = NULL;
+char *markerform;
+char *headerform;
+int headersw = -1;
+
 
 /* mhmisc.c */
-int part_ok (CT, int);
+int part_ok (CT);
+int part_exact (CT);
 int type_ok (CT, int);
 void content_error (char *, CT, char *, ...);
 void flush_errors (void);
 
-/*
- * prototypes
- */
-int show_content_aux (CT, int, char *, char *);
-
 /*
  * static prototypes
  */
 static void show_single_message (CT, char *, int, int, int, struct format *);
 static void DisplayMsgHeader (CT, char *, int);
 static int show_switch (CT, int, int, int, int, struct format *);
-static int show_content (CT, int, int, int, struct format *fmt);
-static int show_content_aux2 (CT, int, char *, char *, int, int, int);
-static int show_text (CT, int, int);
+static int show_content (CT, int, int, int, struct format *);
+static int show_content_aux2 (CT, int, char *, char *, int, int, int, struct format *);
+static int show_text (CT, int, int, struct format *);
 static int show_multi (CT, int, int, int, int, struct format *);
-static int show_multi_internal (CT, int, int, int, int, struct format *fmt);
-static int show_multi_aux (CT, int, char *);
-static int show_message_rfc822 (CT, int);
+static int show_multi_internal (CT, int, int, int, int, struct format *);
+static int show_multi_aux (CT, int, char *, struct format *);
+static int show_message_rfc822 (CT, int, struct format *);
 static int show_partial (CT, int);
 static int show_external (CT, int, int, int, int, struct format *);
 static int parse_display_string (CT, char *, int *, int *, char *, char *,
 				 size_t, int multipart);
 static int convert_content_charset (CT, char **);
+static struct format *compile_header(char *);
 static struct format *compile_marker(char *);
-static void output_marker (CT, struct format *);
+static void output_header (CT, struct format *);
+static void output_marker (CT, struct format *, int);
 static void free_markercomps (void);
 static int pidcheck(int);
 
@@ -72,6 +77,7 @@ static int pidcheck(int);
  * content marker display.
  */
 
+static struct comp *folder_comp = NULL;
 static struct comp *part_comp = NULL;
 static struct comp *ctype_comp = NULL;
 static struct comp *description_comp = NULL;
@@ -92,11 +98,10 @@ static struct param_comp_list *dispo_pc_list = NULL;
  */
 
 void
-show_all_messages (CT *cts, int concatsw, int textonly, int inlineonly,
-		   char *markerform)
+show_all_messages(CT *cts, int concat, int textonly, int inlineonly)
 {
     CT ct, *ctp;
-    struct format *fmt;
+    struct format *hfmt, *mfmt;
 
     /*
      * If form is not specified, then get default form
@@ -106,9 +111,10 @@ show_all_messages (CT *cts, int concatsw, int textonly, int inlineonly,
 	formsw = getcpy (etcpath ("mhl.headers"));
 
     /*
-     * Compile the content marker format line
+     * Compile the content marker and header format lines
      */
-    fmt = compile_marker(markerform);
+    mfmt = compile_marker(markerform);
+    hfmt = compile_header(headerform);
 
     /*
      * If form is "mhl.null", suppress display of header.
@@ -120,13 +126,17 @@ show_all_messages (CT *cts, int concatsw, int textonly, int inlineonly,
 	ct = *ctp;
 
 	/* if top-level type is ok, then display message */
-	if (type_ok (ct, 1))
-	    show_single_message (ct, formsw, concatsw, textonly, inlineonly,
-				 fmt);
+	if (type_ok (ct, 1)) {
+	    if (headersw) output_header(ct, hfmt);
+
+	    show_single_message (ct, formsw, concat, textonly, inlineonly,
+				 mfmt);
+	}
     }
 
     free_markercomps();
-    fmt_free(fmt, 1);
+    fmt_free(hfmt, 1);
+    fmt_free(mfmt, 1);
 }
 
 
@@ -140,7 +150,7 @@ show_single_message (CT ct, char *form, int concatsw, int textonly,
 {
     sigset_t set, oset;
 
-    int status;
+    int status = OK;
 
     /* Allow user executable bit so that temporary directories created by
      * the viewer (e.g., lynx) are going to be accessible */
@@ -196,9 +206,9 @@ DisplayMsgHeader (CT ct, char *form, int concatsw)
     char *file;
 
     vec = argsplit(mhlproc, &file, &vecp);
-    vec[vecp++] = getcpy("-form");
-    vec[vecp++] = getcpy(form);
-    vec[vecp++] = getcpy("-nobody");
+    vec[vecp++] = mh_xstrdup("-form");
+    vec[vecp++] = mh_xstrdup(form);
+    vec[vecp++] = mh_xstrdup("-nobody");
     vec[vecp++] = getcpy(ct->c_file);
 
     /*
@@ -206,10 +216,10 @@ DisplayMsgHeader (CT ct, char *form, int concatsw)
      * then just pass that along.
      */
     if (nomore || concatsw) {
-	vec[vecp++] = getcpy("-nomoreproc");
+	vec[vecp++] = mh_xstrdup("-nomoreproc");
     } else if (progsw) {
-	vec[vecp++] = getcpy("-moreproc");
-	vec[vecp++] = getcpy(progsw);
+	vec[vecp++] = mh_xstrdup("-moreproc");
+	vec[vecp++] = mh_xstrdup(progsw);
     }
     vec[vecp] = NULL;
 
@@ -263,7 +273,7 @@ show_switch (CT ct, int alternate, int concatsw, int textonly, int inlineonly,
 					  inlineonly, fmt);
 
 		case MESSAGE_RFC822:
-		    return show_message_rfc822 (ct, alternate);
+		    return show_message_rfc822 (ct, alternate, fmt);
 
 		/*
 		 * Treat unknown message types as equivalent to
@@ -271,11 +281,11 @@ show_switch (CT ct, int alternate, int concatsw, int textonly, int inlineonly,
 		 */
 		default:
 		    return show_content (ct, alternate, textonly,
-					 inlineonly, fmt);
+		    			 inlineonly, fmt);
 	    }
 
 	case CT_TEXT:
-	    return show_text (ct, alternate, concatsw);
+	    return show_text (ct, alternate, concatsw, fmt);
 
 	case CT_AUDIO:
 	case CT_IMAGE:
@@ -297,7 +307,7 @@ static int
 show_content (CT ct, int alternate, int textonly, int inlineonly,
 	      struct format *fmt)
 {
-    char *cp, buffer[BUFSIZ];
+    char *cp;
     CI ci = &ct->c_ctinfo;
 
     /*
@@ -306,23 +316,16 @@ show_content (CT ct, int alternate, int textonly, int inlineonly,
      */
 
     if (textonly || (inlineonly && !is_inline(ct))) {
-	output_marker(ct, fmt);
+	output_marker(ct, fmt, 1);
 	return OK;
     }
 
-    /* Check for invo_name-show-type/subtype */
-    snprintf (buffer, sizeof(buffer), "%s-show-%s/%s",
-		invo_name, ci->ci_type, ci->ci_subtype);
-    if ((cp = context_find (buffer)) && *cp != '\0')
-	return show_content_aux (ct, alternate, cp, NULL);
-
-    /* Check for invo_name-show-type */
-    snprintf (buffer, sizeof(buffer), "%s-show-%s", invo_name, ci->ci_type);
-    if ((cp = context_find (buffer)) && *cp != '\0')
-	return show_content_aux (ct, alternate, cp, NULL);
+    /* Check for invo_name-show-type[/subtype] */
+    if ((cp = context_find_by_type ("show", ci->ci_type, ci->ci_subtype)))
+	return show_content_aux (ct, alternate, cp, NULL, fmt);
 
     if ((cp = ct->c_showproc))
-	return show_content_aux (ct, alternate, cp, NULL);
+	return show_content_aux (ct, alternate, cp, NULL, fmt);
 
     /* complain if we are not a part of a multipart/alternative */
     if (!alternate)
@@ -337,11 +340,11 @@ show_content (CT ct, int alternate, int textonly, int inlineonly,
  */
 
 int
-show_content_aux (CT ct, int alternate, char *cp, char *cracked)
+show_content_aux (CT ct, int alternate, char *cp, char *cracked, struct format *fmt)
 {
     int fd;
     int xstdin = 0, xlist = 0;
-    char *file, buffer[BUFSIZ];
+    char *file = NULL, buffer[NMH_BUFSIZ];
 
     if (!ct->c_ceopenfnx) {
 	if (!alternate)
@@ -350,11 +353,10 @@ show_content_aux (CT ct, int alternate, char *cp, char *cracked)
 	return NOTOK;
     }
 
-    file = NULL;
     if ((fd = (*ct->c_ceopenfnx) (ct, &file)) == NOTOK)
 	return NOTOK;
     if (ct->c_showproc && !strcmp (ct->c_showproc, "true"))
-	return (alternate ? DONE : OK);
+	return OK;
 
     if (! strcmp(invo_name, "mhshow")  &&
         ct->c_type == CT_TEXT  &&  ct->c_subtype == TEXT_PLAIN) {
@@ -367,10 +369,12 @@ show_content_aux (CT ct, int alternate, char *cp, char *cracked)
             if ((fd = (*ct->c_ceopenfnx) (ct, &file)) == NOTOK)
                 return NOTOK;
         } else {
-            admonish (NULL, "unable to convert character set%s%s to %s",
+            char *charset = content_charset (ct);
+            inform("unable to convert character set%s%s from %s, continuing...",
                       ct->c_partno  ?  " of part "  :  "",
-                      ct->c_partno  ?  ct->c_partno  :  "",
-                      content_charset (ct));
+                      FENDNULL(ct->c_partno),
+                      charset);
+            free (charset);
         }
     }
 
@@ -381,13 +385,13 @@ show_content_aux (CT ct, int alternate, char *cp, char *cracked)
 
     if (parse_display_string (ct, cp, &xstdin, &xlist, file, buffer,
     			      sizeof(buffer) - 1, 0)) {
-	admonish (NULL, "Buffer overflow constructing show command!\n");
+	inform("Buffer overflow constructing show command, continuing...");
 	return NOTOK;
     }
 
 got_command:
     return show_content_aux2 (ct, alternate, cracked, buffer,
-			      fd, xlist, xstdin);
+			      fd, xlist, xstdin, fmt);
 }
 
 
@@ -397,7 +401,7 @@ got_command:
 
 static int
 show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
-                   int fd, int xlist, int xstdin)
+                   int fd, int xlist, int xstdin, struct format *fmt)
 {
     pid_t child_id;
     int i, vecp;
@@ -416,11 +420,8 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
 	    fprintf (stderr, " using command %s\n", buffer);
     }
 
-    if (xlist) {
-	if (ct->c_type == CT_MULTIPART)
-	    list_content (ct, -1, 1, 0, 0, 0);
-	else
-	    list_switch (ct, -1, 1, 0, 0, 0);
+    if (xlist && fmt) {
+	output_marker(ct, fmt, 0);
     }
 
     /*
@@ -434,13 +435,15 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
 	char lastchar = '\n';
 
 	if (fd == NOTOK) {
-	    advise(NULL, "Cannot use NULL command to display content-type "
+	    inform("Cannot use NULL command to display content-type "
 		   "%s/%s", ct->c_ctinfo.ci_type, ct->c_ctinfo.ci_subtype);
 	    return NOTOK;
 	}
 
 	while ((cc = read(fd, readbuf, sizeof(readbuf))) > 0) {
-	    fwrite(readbuf, sizeof(char), cc, stdout);
+	    if ((ssize_t) fwrite(readbuf, sizeof(char), cc, stdout) < cc) {
+		advise ("putline", "fwrite");
+	    }
 	    lastchar = readbuf[cc - 1];
 	}
 
@@ -458,7 +461,7 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
 
 	if (strcmp(invo_name, "mhshow") == 0 && ct->c_type == CT_TEXT &&
 	    ct->c_subtype == TEXT_PLAIN && lastchar != '\n') {
-	    putc('\n', stdout);
+	    putchar('\n');
 	}
 
 	fflush(stdout);
@@ -480,8 +483,11 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
 	    return NOTOK;
 
 	case OK:
-	    if (cracked)
-		chdir (cracked);
+	    if (cracked) {
+		if (chdir (cracked) < 0) {
+		    advise (cracked, "chdir");
+		}
+	    }
 	    if (!xstdin)
 		dup2 (fd, 0);
 	    close (fd);
@@ -491,14 +497,21 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
 	    _exit (-1);
 	    /* NOTREACHED */
 
-	default:
-	    arglist_free(file, vec);
+	default: {
+	    int status;
+            char *display_prog = vecp > 2  &&  vec[2][0] != '\0'
+                /* Copy the real display program name.  This relies on the
+                   specific construction of vec[] by argsplit(). */
+                ?  vec[2]
+                :  NULL;
 
-	    pidcheck (pidXwait (child_id, NULL));
+            pidcheck ((status = pidXwait (child_id, display_prog)));
 
+            arglist_free(file, vec);
 	    if (fd != NOTOK)
 		(*ct->c_ceclosefnx) (ct);
-	    return (alternate ? DONE : OK);
+	    return (alternate ? OK : status);
+        }
     }
 }
 
@@ -508,21 +521,14 @@ show_content_aux2 (CT ct, int alternate, char *cracked, char *buffer,
  */
 
 static int
-show_text (CT ct, int alternate, int concatsw)
+show_text (CT ct, int alternate, int concatsw, struct format *fmt)
 {
     char *cp, buffer[BUFSIZ];
     CI ci = &ct->c_ctinfo;
 
-    /* Check for invo_name-show-type/subtype */
-    snprintf (buffer, sizeof(buffer), "%s-show-%s/%s",
-		invo_name, ci->ci_type, ci->ci_subtype);
-    if ((cp = context_find (buffer)) && *cp != '\0')
-	return show_content_aux (ct, alternate, cp, NULL);
-
-    /* Check for invo_name-show-type */
-    snprintf (buffer, sizeof(buffer), "%s-show-%s", invo_name, ci->ci_type);
-    if ((cp = context_find (buffer)) && *cp != '\0')
-	return show_content_aux (ct, alternate, cp, NULL);
+    /* Check for invo_name-show-type[/subtype] */
+    if ((cp = context_find_by_type ("show", ci->ci_type, ci->ci_subtype)))
+	return show_content_aux (ct, alternate, cp, NULL, fmt);
 
     /*
      * Use default method if content is text/plain, or if
@@ -537,8 +543,8 @@ show_text (CT ct, int alternate, int concatsw)
 	} else
 	    snprintf (buffer, sizeof(buffer), "%%l%s %%F", progsw ? progsw :
 		      moreproc && *moreproc ? moreproc : DEFAULT_PAGER);
-	cp = (ct->c_showproc = add (buffer, NULL));
-	return show_content_aux (ct, alternate, cp, NULL);
+	cp = (ct->c_showproc = mh_xstrdup(buffer));
+	return show_content_aux (ct, alternate, cp, NULL, fmt);
     }
 
     return NOTOK;
@@ -553,22 +559,16 @@ static int
 show_multi (CT ct, int alternate, int concatsw, int textonly, int inlineonly,
 	    struct format *fmt)
 {
-    char *cp, buffer[BUFSIZ];
+    char *cp;
     CI ci = &ct->c_ctinfo;
 
-    /* Check for invo_name-show-type/subtype */
-    snprintf (buffer, sizeof(buffer), "%s-show-%s/%s",
-		invo_name, ci->ci_type, ci->ci_subtype);
-    if ((cp = context_find (buffer)) && *cp != '\0')
-	return show_multi_aux (ct, alternate, cp);
+    /* Check for invo_name-show-type[/subtype] */
+    if ((cp = context_find_by_type ("show", ci->ci_type, ci->ci_subtype)))
+	return show_multi_aux (ct, alternate, cp, fmt);
 
-    /* Check for invo_name-show-type */
-    snprintf (buffer, sizeof(buffer), "%s-show-%s", invo_name, ci->ci_type);
-    if ((cp = context_find (buffer)) && *cp != '\0')
-	return show_multi_aux (ct, alternate, cp);
-
-    if ((cp = ct->c_showproc))
-	return show_multi_aux (ct, alternate, cp);
+    if ((cp = ct->c_showproc)) {
+	return show_multi_aux (ct, alternate, cp, fmt);
+    }
 
     /*
      * Use default method to display this multipart content.  Even
@@ -592,7 +592,10 @@ show_multi_internal (CT ct, int alternate, int concatsw, int textonly,
     int	alternating, nowalternate, result;
     struct multipart *m = (struct multipart *) ct->c_ctparams;
     struct part *part;
-    int any_part_ok;
+    int request_matched;
+    int display_success;
+    int mult_alt_done;
+    int ret;
     CT p;
 
     alternating = 0;
@@ -603,42 +606,58 @@ show_multi_internal (CT ct, int alternate, int concatsw, int textonly,
 	alternating  = 1;
     }
 
-/*
- * alternate   -> we are a part inside an multipart/alternative
- * alternating -> we are a multipart/alternative
- */
+    /*
+     * alternate   -> we are a part inside a multipart/alternative
+     * alternating -> we are a multipart/alternative
+     */
 
-    result = alternate ? NOTOK : OK;
-    any_part_ok = 0;
+    result = NOTOK;
+    request_matched = 0;
+    display_success = 0;
+    mult_alt_done = 0;
 
     for (part = m->mp_parts; part; part = part->mp_next) {
 	p = part->mp_part;
 
-	if (part_ok (p, 1) && type_ok (p, 1)) {
-	    int	inneresult;
+	/* while looking for the right displayable alternative, we
+	 * use a looser search criterion than we do after finding it.
+	 * specifically, while still looking, part_ok() will match
+	 * "parent" parts (e.g.  "-part 2" where 2 is a high-level
+	 * multipart).  after finding it, we use part_exact() to only
+	 * choose a part that was requested explicitly.
+	 */
+	if ((part_exact(p) && type_ok(p, 1)) ||
+		(!mult_alt_done && part_ok (p) && type_ok (p, 1))) {
 
-	    any_part_ok = 1;
+	    int	inneresult;
 
 	    inneresult = show_switch (p, nowalternate, concatsw, textonly,
 				      inlineonly, fmt);
 	    switch (inneresult) {
-		case NOTOK:
+		case NOTOK:  /* hard display error */
+		    request_matched = 1;
 		    if (alternate && !alternating) {
 			result = NOTOK;
 			goto out;
 		    }
 		    continue;
 
-		case OK:
-		case DONE:
+		case DONE:  /* found no match on content type */
+		    continue;
+
+		case OK:  /* display successful */
+		    request_matched = 1;
+		    display_success = 1;
+		    result = OK;
+
+		    /* if we got success on a sub-part of
+		     * multipart/alternative, we're done, unless
+		     * there's a chance an explicit part should be
+		     * matched later in the alternatives.  */
 		    if (alternating) {
-			result = DONE;
-			break;
-		    }
-		    if (alternate) {
+			mult_alt_done = 1;
+		    } else if (alternate) {
 			alternate = nowalternate = 0;
-			if (result == NOTOK)
-			    result = inneresult;
 		    }
 		    continue;
 	    }
@@ -646,15 +665,23 @@ show_multi_internal (CT ct, int alternate, int concatsw, int textonly,
 	}
     }
 
-    if (alternating && !part && any_part_ok) {
+    /* we're supposed to be displaying at least something from a
+     * multipart/alternative.  if we've had parts to consider, and
+     * we've had no success, then we should complain.  we shouldn't
+     * complain if none of the parts matched any -part or -type option.
+     */
+    if (alternating && request_matched && !display_success) {
+	/* if we're ourselves an alternate.  don't complain yet. */
 	if (!alternate)
 	    content_error (NULL, ct, "don't know how to display any of the contents");
 	result = NOTOK;
-	goto out;
     }
 
 out:
-    return result;
+    /* if no parts matched what was requested, there can't have been
+     * any display errors.  we report DONE rather than OK. */
+    ret = request_matched ? result : DONE;
+    return ret;
 }
 
 
@@ -664,12 +691,12 @@ out:
  */
 
 static int
-show_multi_aux (CT ct, int alternate, char *cp)
+show_multi_aux (CT ct, int alternate, char *cp, struct format *fmt)
 {
     /* xstdin is only used in the call to parse_display_string():
        its value is ignored in the function. */
     int xstdin = 0, xlist = 0;
-    char *file, buffer[BUFSIZ];
+    char *file = NULL, buffer[BUFSIZ];
     struct multipart *m = (struct multipart *) ct->c_ctparams;
     struct part *part;
     CT p;
@@ -684,25 +711,24 @@ show_multi_aux (CT ct, int alternate, char *cp)
 	}
 
 	if (p->c_storage == NULL) {
-	    file = NULL;
 	    if ((*p->c_ceopenfnx) (p, &file) == NOTOK)
 		return NOTOK;
 
 	    p->c_storage = add (file, NULL);
 
 	    if (p->c_showproc && !strcmp (p->c_showproc, "true"))
-		return (alternate ? DONE : OK);
+		return OK;
 	    (*p->c_ceclosefnx) (p);
 	}
     }
 
     if (parse_display_string (ct, cp, &xstdin, &xlist, file,
 			      buffer, sizeof(buffer) - 1, 1)) {
-	admonish (NULL, "Buffer overflow constructing show command!\n");
+	inform("Buffer overflow constructing show command, continuing...");
 	return NOTOK;
     }
 
-    return show_content_aux2 (ct, alternate, NULL, buffer, NOTOK, xlist, 0);
+    return show_content_aux2 (ct, alternate, NULL, buffer, NOTOK, xlist, 0, fmt);
 }
 
 
@@ -711,29 +737,22 @@ show_multi_aux (CT ct, int alternate, char *cp)
  */
 
 static int
-show_message_rfc822 (CT ct, int alternate)
+show_message_rfc822 (CT ct, int alternate, struct format *fmt)
 {
-    char *cp, buffer[BUFSIZ];
+    char *cp;
     CI ci = &ct->c_ctinfo;
 
-    /* Check for invo_name-show-type/subtype */
-    snprintf (buffer, sizeof(buffer), "%s-show-%s/%s",
-		invo_name, ci->ci_type, ci->ci_subtype);
-    if ((cp = context_find (buffer)) && *cp != '\0')
-	return show_content_aux (ct, alternate, cp, NULL);
-
-    /* Check for invo_name-show-type */
-    snprintf (buffer, sizeof(buffer), "%s-show-%s", invo_name, ci->ci_type);
-    if ((cp = context_find (buffer)) && *cp != '\0')
-	return show_content_aux (ct, alternate, cp, NULL);
+    /* Check for invo_name-show-type[/subtype] */
+    if ((cp = context_find_by_type ("show", ci->ci_type, ci->ci_subtype)))
+	return show_content_aux (ct, alternate, cp, NULL, fmt);
 
     if ((cp = ct->c_showproc))
-	return show_content_aux (ct, alternate, cp, NULL);
+	return show_content_aux (ct, alternate, cp, NULL, fmt);
 
     /* default method for message/rfc822 */
     if (ct->c_subtype == MESSAGE_RFC822) {
-	cp = (ct->c_showproc = add ("%pshow -file %F", NULL));
-	return show_content_aux (ct, alternate, cp, NULL);
+	cp = (ct->c_showproc = mh_xstrdup("%pshow -file %F"));
+	return show_content_aux (ct, alternate, cp, NULL, fmt);
     }
 
     /* complain if we are not a part of a multipart/alternative */
@@ -829,7 +848,7 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 	    case 'F':
 		/* %f, and stdin is terminal not content */
 		*xstdin = 1;
-		/* and fall... */
+		/* FALLTHRU */
 
 	    case 'f':
 		if (multipart) {
@@ -883,7 +902,7 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 
 	    case 'p':
 		/* No longer supported */
-		/* and fall... */
+		/* FALLTHRU */
 
 	    case 'l':
 		/* display listing prior to displaying content */
@@ -928,8 +947,7 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 		    /* This will get confused if there are multiple %{}'s,
 		       but its real purpose is to avoid doing bad things
 		       above if a closing brace wasn't found. */
-		    admonish(NULL,
-			     "no closing brace for display string escape %s",
+		    inform("no closing brace for display string escape %s, continuing...",
 			     cp);
 		}
 		break;
@@ -978,8 +996,8 @@ parse_display_string (CT ct, char *cp, int *xstdin, int *xlist,
 			   quote. */
 			memmove (pp, pp-1, len+1);
 			*(pp++-1) = '\\';
-			buflen -= 1;
-			bp += 1;
+			buflen--;
+			bp++;
 		    }
 		}
 		/* If pp is still set, that means we ran out of space. */
@@ -1066,10 +1084,12 @@ convert_charset (CT ct, char *dest_charset, int *message_mods) {
 	char *dest_buffer = mh_xmalloc(dest_buffer_size);
         HF hf;
         char *tempfile;
+	int fromutf8 = !strcasecmp(src_charset, "UTF-8");
 
         if ((conv_desc = iconv_open (dest_charset, src_charset)) ==
             (iconv_t) -1) {
-            advise (NULL, "Can't convert %s to %s", src_charset, dest_charset);
+            inform("Can't convert %s to %s", src_charset, dest_charset);
+            free (src_charset);
             return NOTOK;
         }
 
@@ -1077,7 +1097,7 @@ convert_charset (CT ct, char *dest_charset, int *message_mods) {
             adios (NULL, "unable to create temporary file in %s",
                    get_temp_dir());
         }
-        dest = add (tempfile, NULL);
+        dest = mh_xstrdup(tempfile);
 
         if (ct->c_cefile.ce_file) {
             file = &ct->c_cefile.ce_file;
@@ -1134,10 +1154,38 @@ iconv_start:
 			outbytes_before += bumpup;
 			goto iconv_start;
 		    }
+		    if (errno == EINVAL) {
+			/* middle of multi-byte sequence */
+			if (write (fd, dest_buffer, outbytes_before - outbytes) < 0) {
+			    advise (dest, "write");
+			}
+			fseeko (*fp, -inbytes, SEEK_CUR);
+			if (end > 0) bytes_to_read += inbytes;
+			/* inform("convert_charset: EINVAL"); */
+			continue;
+		    }
+		    if (errno == EILSEQ) {
+			/* invalid multi-byte sequence */
+			if (fromutf8) {
+			    for (++ib, --inbytes;
+				 inbytes > 0 &&
+					(((unsigned char) *ib) & 0xc0) == 0x80;
+				 ++ib, --inbytes)
+				continue;
+			} else {
+			    ib++; inbytes--; /* skip it */
+			}
+			(*ob++) = '?'; outbytes --;
+			/* inform("convert_charset: EILSEQ"); */
+			goto iconv_start;
+		    }
+		    inform("convert_charset: errno = %d", errno);
                     status = NOTOK;
                     break;
-                } else {
-                    write (fd, dest_buffer, outbytes_before - outbytes);
+                }
+
+                if (write (fd, dest_buffer, outbytes_before - outbytes) < 0) {
+                    advise (dest, "write");
                 }
             }
 
@@ -1200,13 +1248,14 @@ iconv_start:
 #else  /* ! HAVE_ICONV */
         NMH_UNUSED (message_mods);
 
-        advise (NULL, "Can't convert %s to %s without iconv", src_charset,
+        inform("Can't convert %s to %s without iconv", src_charset,
                 dest_charset);
         errno = ENOSYS;
         status = NOTOK;
 #endif /* ! HAVE_ICONV */
     }
 
+    free (src_charset);
     return status;
 }
 
@@ -1219,21 +1268,22 @@ convert_content_charset (CT ct, char **file) {
     /* Using current locale, see if the content needs to be converted. */
 
     /* content_charset() cannot return NULL. */
-    char *charset = content_charset (ct);
+    char *src_charset = content_charset (ct);
 
-    if (! check_charset (charset, strlen (charset))) {
+    if (! check_charset (src_charset, strlen (src_charset))) {
         int unused = 0;
 
-        char *charset = getcpy (get_charset ());
+        char *dest_charset = getcpy (get_charset ());
 
-        if (convert_charset (ct, charset, &unused) == 0) {
+        if (convert_charset (ct, dest_charset, &unused) == 0) {
             *file = ct->c_cefile.ce_file;
         } else {
             status = NOTOK;
         }
 
-        free (charset);
+        free (dest_charset);
     }
+    free (src_charset);
 #else  /* ! HAVE_ICONV */
     NMH_UNUSED (ct);
     NMH_UNUSED (file);
@@ -1246,12 +1296,37 @@ convert_content_charset (CT ct, char **file) {
  * Compile our format string and save any parameters we care about.
  */
 
-#define DEFAULT_MARKER "[ part %{part} - %{content-type} - %<{description}" \
-		       "%{description}%?{cdispo-filename}%{cdispo-filename}" \
-		       "%|%{ctype-name}%> ]"
+#define DEFAULT_HEADER "[ Message %{folder}%<{folder}:%>%(msg) ]"
+#define DEFAULT_MARKER "[ part %{part} - %{content-type} - " \
+		       "%<{description}%{description}" \
+		         "%?{cdispo-filename}%{cdispo-filename}" \
+		         "%|%{ctype-name}%>  " \
+		       "%(kilo(size))B %<(unseen)\\(suppressed\\)%> ]"
 
 static struct format *
-compile_marker(char *markerform)
+compile_header(char *form)
+{
+    struct format *fmt;
+    char *fmtstring;
+    struct comp *comp = NULL;
+    unsigned int bucket;
+
+    fmtstring = new_fs(form, NULL, DEFAULT_HEADER);
+
+    (void) fmt_compile(fmtstring, &fmt, 1);
+    free_fs();
+
+    while ((comp = fmt_nextcomp(comp, &bucket)) != NULL) {
+	if (strcasecmp(comp->c_name, "folder") == 0) {
+	    folder_comp = comp;
+	}
+    }
+
+    return fmt;
+}
+
+static struct format *
+compile_marker(char *form)
 {
     struct format *fmt;
     char *fmtstring;
@@ -1259,10 +1334,10 @@ compile_marker(char *markerform)
     unsigned int bucket;
     struct param_comp_list *pc_entry;
 
-    fmtstring = new_fs(markerform, NULL, DEFAULT_MARKER);
+    fmtstring = new_fs(form, NULL, DEFAULT_MARKER);
 
     (void) fmt_compile(fmtstring, &fmt, 1);
-    free(fmtstring);
+    free_fs();
 
     /*
      * Things we care about:
@@ -1286,15 +1361,15 @@ compile_marker(char *markerform)
 	    dispo_comp = comp;
 	} else if (strncasecmp(comp->c_name, "ctype-", 6) == 0 &&
 		   strlen(comp->c_name) > 6) {
-	    pc_entry = mh_xmalloc(sizeof(*pc_entry));
-	    pc_entry->param = getcpy(comp->c_name + 6);
+	    NEW(pc_entry);
+	    pc_entry->param = mh_xstrdup(comp->c_name + 6);
 	    pc_entry->comp = comp;
 	    pc_entry->next = ctype_pc_list;
 	    ctype_pc_list = pc_entry;
 	} else if (strncasecmp(comp->c_name, "cdispo-", 7) == 0 &&
 		   strlen(comp->c_name) > 7) {
-	    pc_entry = mh_xmalloc(sizeof(*pc_entry));
-	    pc_entry->param = getcpy(comp->c_name + 7);
+	    NEW(pc_entry);
+	    pc_entry->param = mh_xstrdup(comp->c_name + 7);
 	    pc_entry->comp = comp;
 	    pc_entry->next = dispo_pc_list;
 	    dispo_pc_list = pc_entry;
@@ -1309,11 +1384,45 @@ compile_marker(char *markerform)
  */
 
 static void
-output_marker(CT ct, struct format *fmt)
+output_header(CT ct, struct format *fmt)
 {
-    char outbuf[BUFSIZ];
+    charstring_t outbuf = charstring_create (BUFSIZ);
+    int dat[5] = { 0 };
+    char *endp;
+    int message = 0;
+
+    if (folder_comp)
+	folder_comp->c_text = getcpy(folder);
+
+    if (ct->c_file && *ct->c_file) {
+	message = strtol(ct->c_file, &endp, 10);
+	if (*endp) message = 0;
+	dat[0] = message;
+    }
+
+    /* it would be nice to populate dat[2], for %(size) here,
+     * but it's not available.  it might also be nice to know
+     * if the message originally had any mime parts or not -- but
+     * there's also no record of that.  (except for MIME-version:)
+     */
+
+    fmt_scan(fmt, outbuf, BUFSIZ, dat, NULL);
+
+    fputs(charstring_buffer (outbuf), stdout);
+    charstring_free (outbuf);
+
+    fmt_freecomptext();
+}
+
+static void
+output_marker(CT ct, struct format *fmt, int hidden)
+{
+    charstring_t outbuf = charstring_create (BUFSIZ);
     struct param_comp_list *pcentry;
-    int dat[5];
+    int partsize;
+    int message = 0;
+    char *endp;
+    int dat[5] = { 0 };
 
     /*
      * Grab any items we care about.
@@ -1325,15 +1434,15 @@ output_marker(CT ct, struct format *fmt)
     }
 
     if (part_comp && ct->c_partno) {
-	part_comp->c_text = getcpy(ct->c_partno);
+	part_comp->c_text = mh_xstrdup(ct->c_partno);
     }
 
     if (description_comp && ct->c_descr) {
-	description_comp->c_text = getcpy(ct->c_descr);
+	description_comp->c_text = mh_xstrdup(ct->c_descr);
     }
 
     if (dispo_comp && ct->c_dispo_type) {
-	dispo_comp->c_text = getcpy(ct->c_dispo_type);
+	dispo_comp->c_text = mh_xstrdup(ct->c_dispo_type);
     }
 
     for (pcentry = ctype_pc_list; pcentry != NULL; pcentry = pcentry->next) {
@@ -1346,9 +1455,28 @@ output_marker(CT ct, struct format *fmt)
 					  pcentry->param, '?', 0);
     }
 
-    fmt_scan(fmt, outbuf, sizeof(outbuf), sizeof(outbuf), dat, NULL);
+    if (ct->c_cesizefnx)
+	partsize = (*ct->c_cesizefnx) (ct);
+    else
+	partsize = ct->c_end - ct->c_begin;
 
-    fputs(outbuf, stdout);
+    if (ct->c_file && *ct->c_file) {
+	message = strtol(ct->c_file, &endp, 10);
+	if (*endp) message = 0;
+	dat[0] = message;
+    }
+    dat[2] = partsize;
+
+    /* make the part's hidden aspect available by overloading the
+     * %(unseen) function.  make the part's size available via %(size).
+     * see comments in h/fmt_scan.h.
+     */
+    dat[4] = hidden;
+
+    fmt_scan(fmt, outbuf, BUFSIZ, dat, NULL);
+
+    fputs(charstring_buffer (outbuf), stdout);
+    charstring_free (outbuf);
 
     fmt_freecomptext();
 }
@@ -1362,6 +1490,7 @@ free_markercomps(void)
 {
     struct param_comp_list *pc_entry, *pc2;
 
+    folder_comp = NULL;
     part_comp = NULL;
     ctype_comp = NULL;
     description_comp = NULL;
