@@ -1,6 +1,4 @@
-
-/*
- * mhstoresbr.c -- routines to save/store the contents of MIME messages
+/* mhstoresbr.c -- routines to save/store the contents of MIME messages
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -9,13 +7,16 @@
 
 #include <h/mh.h>
 #include <fcntl.h>
-#include <h/signals.h>
 #include <h/md5.h>
 #include <h/mts.h>
 #include <h/tws.h>
+#include <h/fmt_scan.h>
 #include <h/mime.h>
 #include <h/mhparse.h>
 #include <h/utils.h>
+#include "mhshowsbr.h"
+#include "../sbr/m_maildir.h"
+#include "../sbr/m_mktemp.h"
 
 enum clobber_policy_t {
   NMH_CLOBBER_ALWAYS = 0,
@@ -41,12 +42,13 @@ struct mhstoreinfo {
     enum clobber_policy_t clobber_policy;  /* -clobber selection */
 };
 
-typedef struct mhstoreinfo *mhstoreinfo_t;
+static bool use_param_as_filename(const char *p);
 
 mhstoreinfo_t
 mhstoreinfo_create (CT *ct, char *pwd, const char *csw, int asw, int vsw) {
-    mhstoreinfo_t info = mh_xmalloc (sizeof *info);
+    mhstoreinfo_t info;
 
+    NEW(info);
     info->cts = ct;
     info->cwd = pwd;
     info->autosw = asw;
@@ -61,6 +63,7 @@ mhstoreinfo_create (CT *ct, char *pwd, const char *csw, int asw, int vsw) {
 void
 mhstoreinfo_free (mhstoreinfo_t info) {
     free (info->cwd);
+    free (info->dir);
     free (info);
 }
 
@@ -78,18 +81,10 @@ typedef int (*qsort_comp) (const void *, const void *);
 
 
 /* mhmisc.c */
-int part_ok (CT, int);
+int part_ok (CT);
 int type_ok (CT, int);
 void flush_errors (void);
 
-/* mhshowsbr.c */
-int show_content_aux (CT, int, char *, char *);
-
-/*
- * prototypes
- */
-void store_all_messages (mhstoreinfo_t);
-
 /*
  * static prototypes
  */
@@ -125,7 +120,7 @@ store_all_messages (mhstoreinfo_t info)
      * store any contents.
      */
     if ((cp = context_find (nmhstorage)) && *cp)
-	info->dir = getcpy (cp);
+	info->dir = mh_xstrdup(cp);
     else
 	info->dir = getcpy (info->cwd);
 
@@ -296,15 +291,14 @@ store_multi (CT ct, mhstoreinfo_t info)
     for (part = m->mp_parts; part; part = part->mp_next) {
 	CT  p = part->mp_part;
 
-	if (part_ok (p, 1) && type_ok (p, 1)) {
+	if (part_ok (p) && type_ok (p, 1)) {
 	    if (ct->c_storage) {
 		/* Support mhstore -outfile.  The MIME parser doesn't
 		   load c_storage, so we know that p->c_storage is
 		   NULL here. */
-		p->c_storage = ct->c_storage;
+		p->c_storage = mh_xstrdup(ct->c_storage);
 	    }
 	    result = store_switch (p, info);
-	    p->c_storage = NULL;
 
 	    if (result == OK && ct->c_subtype == MULTI_ALTERNATE)
 		break;
@@ -351,13 +345,11 @@ store_partial (CT ct, mhstoreinfo_t info)
     }
 
     if (hi == 0) {
-	advise (NULL, "missing (at least) last part of multipart message");
+	inform("missing (at least) last part of multipart message");
 	return NOTOK;
     }
 
-    if ((base = (CT *) calloc ((size_t) (i + 1), sizeof(*base))) == NULL)
-	adios (NULL, "out of memory");
-
+    base = mh_xcalloc(i + 1, sizeof *base);
     ctq = base;
     for (ctp = info->cts; *ctp; ctp++) {
 	p = *ctp;
@@ -376,22 +368,21 @@ store_partial (CT ct, mhstoreinfo_t info)
     for (ctq = base; *ctq; ctq++) {
 	p = *ctq;
 	pm = (struct partial *) p->c_ctparams;
-	if (pm->pm_marked != cur) {
-	    if (pm->pm_marked == cur - 1) {
-		admonish (NULL,
-			  "duplicate part %d of %d part multipart message",
-			  pm->pm_marked, hi);
-		continue;
-	    }
+	if (pm->pm_marked == cur) {
+	    cur++;
+            continue;
+        }
+
+        if (pm->pm_marked == cur - 1) {
+            inform("duplicate part %d of %d part multipart message, continuing...",
+                      pm->pm_marked, hi);
+            continue;
+        }
 
 missing_part:
-	    advise (NULL,
-		    "missing %spart %d of %d part multipart message",
-		    cur != hi ? "(at least) " : "", cur, hi);
-	    goto losing;
-	}
-        else
-	    cur++;
+        inform("missing %spart %d of %d part multipart message",
+            cur != hi ? "(at least) " : "", cur, hi);
+        goto losing;
     }
     if (hi != --cur) {
 	cur = hi;
@@ -407,7 +398,7 @@ missing_part:
     ct = *ctq++;
     if (store_content (ct, NULL, info) == NOTOK) {
 losing:
-	free ((char *) base);
+	free(base);
 	return NOTOK;
     }
 
@@ -417,7 +408,7 @@ losing:
 	    goto losing;
     }
 
-    free ((char *) base);
+    free(base);
     return OK;
 }
 
@@ -443,16 +434,11 @@ store_external (CT ct, mhstoreinfo_t info)
     if (info->autosw) {
 	char *cp;
 
-	if ((cp = e->eb_name)
-	    && *cp != '/'
-	    && *cp != '.'
-	    && *cp != '|'
-	    && *cp != '!'
-	    && !strchr (cp, '%')) {
+	if ((cp = e->eb_name) && use_param_as_filename(cp)) {
 	    if (!ct->c_storeproc)
-		ct->c_storeproc = add (cp, NULL);
+		ct->c_storeproc = mh_xstrdup(cp);
 	    if (!p->c_storeproc)
-		p->c_storeproc = add (cp, NULL);
+		p->c_storeproc = mh_xstrdup(cp);
 	}
     }
 
@@ -468,10 +454,9 @@ store_external (CT ct, mhstoreinfo_t info)
     if (ct->c_storage) {
 	/* Support mhstore -outfile.  The MIME parser doesn't load
 	   c_storage, so we know that p->c_storage is NULL here. */
-	p->c_storage = ct->c_storage;
+	p->c_storage = mh_xstrdup(ct->c_storage);
     }
     result = store_switch (p, info);
-    p->c_storage = NULL;
 
     p->c_partno = NULL;
     return result;
@@ -550,7 +535,7 @@ store_content (CT ct, CT p, mhstoreinfo_t info)
 
 		/* record the folder name */
 		if (p->c_folder) {
-		    ct->c_folder = add (p->c_folder, NULL);
+		    ct->c_folder = mh_xstrdup(p->c_folder);
 		}
 	    }
 	    goto got_filename;
@@ -569,13 +554,9 @@ store_content (CT ct, CT p, mhstoreinfo_t info)
     if ((cp = ct->c_storeproc) == NULL || *cp == '\0') {
 	CI ci = &ct->c_ctinfo;
 
-	snprintf (buffer, sizeof(buffer), "%s-store-%s/%s",
-		invo_name, ci->ci_type, ci->ci_subtype);
-	if ((cp = context_find (buffer)) == NULL || *cp == '\0') {
-	    snprintf (buffer, sizeof(buffer), "%s-store-%s", invo_name, ci->ci_type);
-	    if ((cp = context_find (buffer)) == NULL || *cp == '\0') {
-		cp = ct->c_type == CT_MESSAGE ? "+" : "%m%P.%s";
-	    }
+	cp = context_find_by_type ("store", ci->ci_type, ci->ci_subtype);
+	if (cp == NULL) {
+	    cp = ct->c_type == CT_MESSAGE ? "+" : "%m%P.%s";
 	}
     }
 
@@ -592,7 +573,7 @@ store_content (CT ct, CT p, mhstoreinfo_t info)
 		adios(NULL, "unable to create temporary file in %s",
 		      get_temp_dir());
 	    }
-	    ct->c_storage = add (tmpfilenam, NULL);
+	    ct->c_storage = mh_xstrdup(tmpfilenam);
 
 	    /* Get the folder name */
 	    if (cp[1])
@@ -604,7 +585,7 @@ store_content (CT ct, CT p, mhstoreinfo_t info)
 	    create_folder(m_mailpath(folder), 0, exit);
 
 	    /* Record the folder name */
-	    ct->c_folder = add (folder, NULL);
+	    ct->c_folder = mh_xstrdup(folder);
 
 	    if (cp[1])
 		free (folder);
@@ -623,16 +604,16 @@ store_content (CT ct, CT p, mhstoreinfo_t info)
 	 * content to standard input of a command and return.
 	 */
 	if (buffer[0] == '|' || buffer[0] == '!')
-	    return show_content_aux (ct, 0, buffer + 1, info->dir);
+	    return show_content_aux (ct, 0, buffer + 1, info->dir, NULL);
 
         /* record the filename */
-	if ((ct->c_storage = clobber_check (add (buffer, NULL), info)) ==
+	if ((ct->c_storage = clobber_check (mh_xstrdup(buffer), info)) ==
 	    NULL) {
 	    return NOTOK;
 	}
     } else {
 	/* The output filename was explicitly specified, so use it. */
-	if ((ct->c_storage = clobber_check (add (ct->c_storage, NULL), info)) ==
+	if ((ct->c_storage = clobber_check (ct->c_storage, info)) ==
 	    NULL) {
 	    return NOTOK;
 	}
@@ -667,7 +648,7 @@ got_filename:
             if (first_partial)
                 fprintf (stderr, "reassembling partials ");
             if (last_partial)
-                fprintf (stderr, "%s", ct->c_file);
+                fputs(ct->c_file, stderr);
             else
                 fprintf (stderr, "%s,", ct->c_file);
         } else {
@@ -691,7 +672,7 @@ got_filename:
                 int cwdlen = strlen (info->cwd);
 
                 fprintf (stderr, " as file %s\n",
-                         strncmp (ct->c_storage, info->cwd, cwdlen)
+                         !has_prefix(ct->c_storage, info->cwd)
                          || ct->c_storage[cwdlen] != '/'
                          ? ct->c_storage : ct->c_storage + cwdlen + 1);
             }
@@ -726,7 +707,7 @@ output_content_file (CT ct, int appending)
 	int cc, fd;
 
 	if (!ct->c_ceopenfnx) {
-	    advise (NULL, "don't know how to decode part %s of message %s",
+	    inform("don't know how to decode part %s of message %s",
 		    ct->c_partno, ct->c_file);
 	    return NOTOK;
 	}
@@ -790,7 +771,9 @@ losing:
 		    break;
 
 		default:
-		    fwrite (buffer, sizeof(*buffer), cc, fp);
+		    if ((int) fwrite (buffer, sizeof(*buffer), cc, fp) < cc) {
+			advise ("output_content_file", "fwrite");
+		    }
 		    continue;
 	    }
 	    break;
@@ -850,7 +833,7 @@ losing:
 	}
     }
 
-    while (fgets (buffer, sizeof(buffer) - 1, ct->c_fp)) {
+    while (fgets (buffer, sizeof buffer, ct->c_fp)) {
 	if ((pos += strlen (buffer)) > last) {
 	    int diff;
 
@@ -921,9 +904,9 @@ output_content_folder (char *folder, char *filename)
     /* Read the folder. */
     if ((mp = folder_read (folder, 0))) {
 	/* Link file into folder */
-	msgnum = folder_addmsg (&mp, filename, 0, 0, 0, 0, (char *)0);
+	msgnum = folder_addmsg (&mp, filename, 0, 0, 0, 0, NULL);
     } else {
-	advise (NULL, "unable to read folder %s", folder);
+	inform("unable to read folder %s", folder);
 	return NOTOK;
     }
 
@@ -968,7 +951,9 @@ parse_format_string (CT ct, char *cp, char *buffer, int buflen, char *dir)
      * appropriate directory.
      */
     if (*cp != '/' && *cp != '|' && *cp != '!') {
-	snprintf (bp, buflen, "%s/", dir[1] ? dir : "");
+        if (!strcmp(dir, "/"))
+            dir = ""; /* Don't start with "//". */
+	snprintf (bp, buflen, "%s/", dir);
 	len = strlen (bp);
 	bp += len;
 	buflen -= len;
@@ -989,7 +974,8 @@ parse_format_string (CT ct, char *cp, char *buffer, int buflen, char *dir)
 			*bp = '\0';
 			buflen--;
 			continue;
-		    } else {
+		    }
+                    {
 			PM pm;
 			char *s = "";
 
@@ -1016,7 +1002,7 @@ parse_format_string (CT ct, char *cp, char *buffer, int buflen, char *dir)
 		    break;
 
 		case 'p':
-		    /* insert part number withouth leading dot */
+		    /* insert part number without leading dot */
 		    if (ct->c_partno)
 			strncpy (bp, ct->c_partno, buflen);
 		    break;
@@ -1083,18 +1069,13 @@ get_storeproc (CT ct)
      * use that (RFC-2183).
      */
     if (ct->c_dispo) {
-	if ((cp = get_param(ct->c_dispo_first, "filename", '_', 0))
-		&& *cp != '/'
-		&& *cp != '.'
-		&& *cp != '|'
-		&& *cp != '!'
-		&& !strchr (cp, '%')) {
-		ct->c_storeproc = add (cp, NULL);
+	if ((cp = get_param(ct->c_dispo_first, "filename", '_', 0)) &&
+            use_param_as_filename(cp)) {
+		ct->c_storeproc = mh_xstrdup(cp);
 		free(cp);
 		return;
 	}
-	if (cp)
-	    free(cp);
+        mh_xfree(cp);
     }
 
     /*
@@ -1103,17 +1084,12 @@ get_storeproc (CT ct)
      * the storeproc.
      */
     ci = &ct->c_ctinfo;
-    if ((cp = get_param(ci->ci_first_pm, "name", '_', 0))
-	  && *cp != '/'
-	  && *cp != '.'
-	  && *cp != '|'
-	  && *cp != '!'
-	  && !strchr (cp, '%')) {
-	    ct->c_storeproc = add (cp, NULL);
+    if ((cp = get_param(ci->ci_first_pm, "name", '_', 0)) &&
+        use_param_as_filename(cp)) {
+	    ct->c_storeproc = mh_xstrdup(cp);
 
     }
-    if (cp)
-	free(cp);
+    mh_xfree(cp);
 }
 
 
@@ -1153,19 +1129,22 @@ static
 enum clobber_policy_t
 clobber_policy (const char *value) {
   if (value == NULL  ||  ! strcasecmp (value, "always")) {
-  } else if (! strcasecmp (value, "auto")) {
+    return NMH_CLOBBER_ALWAYS;
+  }
+  if (! strcasecmp (value, "auto")) {
     return NMH_CLOBBER_AUTO;
-  } else if (! strcasecmp (value, "suffix")) {
+  }
+  if (! strcasecmp (value, "suffix")) {
     return NMH_CLOBBER_SUFFIX;
-  } else if (! strcasecmp (value, "ask")) {
+  }
+  if (! strcasecmp (value, "ask")) {
     return NMH_CLOBBER_ASK;
-  } else if (! strcasecmp (value, "never")) {
+  }
+  if (! strcasecmp (value, "never")) {
     return NMH_CLOBBER_NEVER;
-  } else {
-    adios (NULL, "invalid argument, %s, to clobber", value);
   }
 
-  return NMH_CLOBBER_ALWAYS;
+  adios (NULL, "invalid argument, %s, to clobber", value);
 }
 
 
@@ -1200,7 +1179,7 @@ next_version (char *file, enum clobber_policy_t clobber_policy) {
 
       default:
         /* Should never get here. */
-        advise (NULL, "will not overwrite %s, invalid clobber policy", buffer);
+        inform("will not overwrite %s, invalid clobber policy", buffer);
         free (buffer);
         return NULL;
     }
@@ -1221,7 +1200,7 @@ next_version (char *file, enum clobber_policy_t clobber_policy) {
   free (file);
 
   if (version >= max_versions) {
-    advise (NULL, "will not overwrite %s, too many versions", buffer);
+    inform("will not overwrite %s, too many versions", buffer);
     free (buffer);
     buffer = NULL;
   }
@@ -1245,17 +1224,23 @@ clobber_check (char *original_file, mhstoreinfo_t info) {
   char *cwd = NULL;
   int check_again;
 
+  if (! strcmp (original_file, "-")) {
+      return original_file;
+  }
+
   if (info->clobber_policy == NMH_CLOBBER_ASK) {
     /* Save cwd for possible use in loop below. */
     char *slash;
 
-    cwd = add (original_file, NULL);
+    cwd = mh_xstrdup(original_file);
     slash = strrchr (cwd, '/');
 
     if (slash) {
       *slash = '\0';
     } else {
-      /* original_file wasn't a full path, which shouldn't happen. */
+      /* original_file isn't a full path, which should only happen if
+         it is -. */
+      free (cwd);
       cwd = NULL;
     }
   }
@@ -1293,11 +1278,11 @@ clobber_check (char *original_file, mhstoreinfo_t info) {
           if (isatty (fileno (stdin))) {
             char *prompt =
               concat ("Overwrite \"", file, "\" [y/n/rename]? ", NULL);
-            ans = getans (prompt, answer);
+            ans = read_switch_multiword (prompt, answer);
             free (prompt);
           } else {
             /* Overwrite, that's what nmh used to do.  And warn. */
-            advise (NULL, "-clobber ask but no tty, so overwrite %s", file);
+            inform("-clobber ask but no tty, so overwrite %s", file);
             break;
           }
 
@@ -1317,20 +1302,17 @@ clobber_check (char *original_file, mhstoreinfo_t info) {
                 file = NULL;
                 ++info->files_not_clobbered;
               } else {
-                char *newline = strchr (buf, '\n');
-                if (newline) {
-                  *newline = '\0';
-                }
+                trim_suffix_c(buf, '\n');
               }
 
               free (file);
 
               if (buf[0] == '/') {
                 /* Full path, use it. */
-                file = add (buf, NULL);
+                file = mh_xstrdup(buf);
               } else {
                 /* Relative path. */
-                file = cwd  ?  concat (cwd, "/", buf, NULL)  :  add (buf, NULL);
+                file = cwd  ?  concat (cwd, "/", buf, NULL)  :  mh_xstrdup(buf);
               }
 
               check_again = 1;
@@ -1344,7 +1326,7 @@ clobber_check (char *original_file, mhstoreinfo_t info) {
         if (stat (file, &st) == OK) {
           /* Keep count of files that would have been clobbered,
              and return that as process exit status. */
-          advise (NULL, "will not overwrite %s with -clobber never", file);
+          inform("will not overwrite %s with -clobber never", file);
           free (file);
           file = NULL;
           ++info->files_not_clobbered;
@@ -1355,12 +1337,17 @@ clobber_check (char *original_file, mhstoreinfo_t info) {
     original_file = file;
   } while (check_again);
 
-  if (cwd) {
-    free (cwd);
-  }
+  free (cwd);
 
   return file;
 }
 
+static bool use_param_as_filename(const char *p)
+{
+    /* Preserve result of original test that considered an empty string
+     * OK. */
+    return !*p || (!strchr("/.|!", *p) && !strchr(p, '%'));
+}
+
 /* -clobber support */
 /******************************************************************************/
