@@ -1,6 +1,4 @@
-
-/*
- * msgchk.c -- check for mail
+/* msgchk.c -- check for mail
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -10,6 +8,7 @@
 #include <h/mh.h>
 #include <h/mts.h>
 #include <h/tws.h>
+#include <h/utils.h>
 #include <pwd.h>
 
 #include <h/popsbr.h>
@@ -20,6 +19,12 @@
 # define SASLminc(a)  0
 #endif
 
+#ifndef TLS_SUPPORT
+# define TLSminc(a) (a)
+#else
+# define TLSminc(a)  0
+#endif
+
 #define MSGCHK_SWITCHES \
     X("date", 0, DATESW) \
     X("nodate", 0, NDATESW) \
@@ -30,9 +35,15 @@
     X("port name/number", 0, PORTSW) \
     X("version", 0, VERSIONSW) \
     X("help", 0, HELPSW) \
-    X("snoop", -5, SNOOPSW) \
-    X("sasl", SASLminc(-4), SASLSW) \
-    X("saslmech", SASLminc(-5), SASLMECHSW) \
+    X("snoop", 0, SNOOPSW) \
+    X("sasl", SASLminc(4), SASLSW) \
+    X("nosasl", SASLminc(6), NOSASLSW) \
+    X("saslmech", SASLminc(5), SASLMECHSW) \
+    X("authservice", SASLminc(0), AUTHSERVICESW) \
+    X("initialtls", TLSminc(-10), INITTLSSW) \
+    X("notls", TLSminc(-5), NOTLSSW) \
+    X("certverify", TLSminc(-10), CERTVERSW) \
+    X("nocertverify", TLSminc(-12), NOCERTVERSW) \
     X("proxy command", 0, PROXYSW) \
 
 #define X(sw, minchars, id) id,
@@ -71,23 +82,23 @@ DEFINE_SWITCH_ARRAY(MSGCHK, switches);
 static int donote (char *, int);
 static int checkmail (char *, char *, int, int, int);
 static int remotemail (char *, char *, char *, char *, int, int, int, int,
-		       char *);
+		       char *, int, const char *);
 
 
 int
 main (int argc, char **argv)
 {
     int datesw = 1, notifysw = NT_ALL;
-    int status = 0, sasl = 0;
+    int status = 0, sasl = 0, tls = 0, noverify = 0;
     int snoop = 0, vecp = 0;
     char *cp, *host = NULL, *port = NULL, *user = NULL, *proxy = NULL;
-    char buf[BUFSIZ], *saslmech = NULL; 
+    char buf[BUFSIZ], *saslmech = NULL, *auth_svc = NULL;
     char **argp, **arguments, *vec[MAXVEC];
     struct passwd *pw;
 
     if (nmh_init(argv[0], 1)) { return 1; }
 
-    mts_init (invo_name);
+    mts_init ();
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -155,11 +166,40 @@ main (int argc, char **argv)
 		    sasl++;
 		    continue;
 		
+		case NOSASLSW:
+		    sasl = 0;
+		    continue;
+
 		case SASLMECHSW:
 		    if (!(saslmech = *argp++) || *saslmech == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
 		    continue;
 
+		case INITTLSSW:
+		    tls++;
+		    continue;
+
+		case NOTLSSW:
+		    tls = 0;
+		    continue;
+
+		case CERTVERSW:
+		    noverify = 0;
+		    continue;
+
+		case NOCERTVERSW:
+		    noverify++;
+		    continue;
+
+		case AUTHSERVICESW:
+#ifdef OAUTH_SUPPORT
+		    if (!(auth_svc = *argp++) || *auth_svc == '-')
+			adios (NULL, "missing argument to %s", argp[-2]);
+#else
+		    adios (NULL, "not built with OAuth support");
+#endif
+		    continue;
+
 		case PROXYSW:
 		    if (!(proxy = *argp++) || *proxy == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
@@ -190,13 +230,21 @@ main (int argc, char **argv)
 	vec[vecp] = NULL;
 
     if (host) {
+	int tlsflag = 0;
+
+	if (tls)
+	    tlsflag |= P_INITTLS;
+
+	if (noverify)
+	    tlsflag |= P_NOVERIFY;
+
 	if (vecp == 0) {
 	    status = remotemail (host, port, user, proxy, notifysw, 1,
-				 snoop, sasl, saslmech);
+				 snoop, sasl, saslmech, tlsflag, auth_svc);
 	} else {
 	    for (vecp = 0; vec[vecp]; vecp++)
 		status += remotemail (host, port, vec[vecp], proxy, notifysw, 0,
-				      snoop, sasl, saslmech);
+				      snoop, sasl, saslmech, tlsflag, auth_svc);
 	}
     } else {
 	if (user == NULL) user = getusername ();
@@ -216,7 +264,7 @@ main (int argc, char **argv)
 		if ((pw = getpwnam (vec[vecp])))
 		    status += checkmail (pw->pw_name, pw->pw_dir, datesw, notifysw, 0);
 		else
-		    advise (NULL, "no such user as %s", vec[vecp]);
+		    inform("no such user as %s", vec[vecp]);
 	    }
 	}
     }		/* host == NULL */
@@ -310,7 +358,7 @@ checkmail (char *user, char *home, int datesw, int notifysw, int personal)
 	if (datesw && st.st_atime)
 	    printf ("; last read on %s", dtime (&st.st_atime, 1));
     if (notifysw)
-	printf ("\n");
+	putchar('\n');
 
     return status;
 }
@@ -320,18 +368,27 @@ extern char response[];
 
 static int
 remotemail (char *host, char *port, char *user, char *proxy, int notifysw,
-	    int personal, int snoop, int sasl, char *saslmech)
+	    int personal, int snoop, int sasl, char *saslmech, int tls,
+	    const char *auth_svc)
 {
     int nmsgs, nbytes, status;
-    struct nmh_creds creds = { 0, 0, 0 };
+
+    if (auth_svc == NULL) {
+	if (saslmech  &&  ! strcasecmp(saslmech, "xoauth2")) {
+	    adios (NULL, "must specify -authservice with -saslmech xoauth2");
+	}
+    } else {
+	if (user == NULL) {
+	    adios (NULL, "must specify -user with -saslmech xoauth2");
+	}
+    }
 
     /* open the POP connection */
-    nmh_get_credentials (host, user, sasl, &creds);
-    if (pop_init (host, port, creds.user, creds.password, proxy, snoop, sasl,
-		  saslmech) == NOTOK
+    if (pop_init (host, port, user, proxy, snoop, sasl, saslmech, tls,
+		  auth_svc) == NOTOK
 	    || pop_stat (&nmsgs, &nbytes) == NOTOK     /* check for messages  */
 	    || pop_quit () == NOTOK) {                 /* quit POP connection */
-	advise (NULL, "%s", response);
+	inform("%s", response);
 	return 1;
     }
 
@@ -343,7 +400,7 @@ remotemail (char *host, char *port, char *user, char *proxy, int notifysw,
 		printf ("%s has ", user);
 
 	    printf ("%d message%s (%d bytes)",
-		    nmsgs, nmsgs != 1 ? "s" : "", nbytes);
+		    nmsgs, PLURALS(nmsgs), nbytes);
 	}
 	else
 	    notifysw = 0;
