@@ -1,6 +1,4 @@
-
-/*
- * pick.c -- search for messages by content
+/* pick.c -- search for messages by content
  *
  * This code is Copyright (c) 2002, 2008, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -11,8 +9,10 @@
 #include <h/tws.h>
 #include <h/picksbr.h>
 #include <h/utils.h>
+#include "../sbr/m_maildir.h"
 
 #define PICK_SWITCHES \
+    X("reverse", 0, REVSW) \
     X("and", 0, ANDSW) \
     X("or", 0, ORSW) \
     X("not", 0, NOTSW) \
@@ -36,6 +36,7 @@
     X("nozero", 0, NZEROSW) \
     X("list", 0, LISTSW) \
     X("nolist", 0, NLISTSW) \
+    X("debug", 0, DEBUGSW) \
     X("version", 0, VERSIONSW) \
     X("help", 0, HELPSW) \
 
@@ -64,12 +65,19 @@ main (int argc, char **argv)
     struct msgs_array msgs = { 0, 0, NULL };
     struct msgnum_array nums = { 0, 0, NULL };
     struct msgs *mp, *mp2;
-    register FILE *fp;
+    FILE *fp;
+    int debug = 0;
+    int reverse = 0;
+    int start, end, inc;
 
     if (nmh_init(argv[0], 1)) { return 1; }
 
     done=putzero_done;
 
+    /* Deprecated.  Use -debug instead. */
+    if ((cp = getenv ("MHPDEBUG")) && *cp)
+	++debug;
+
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
@@ -98,6 +106,10 @@ main (int argc, char **argv)
 		listsw = 0;	/* HACK */
 		done (0);
 
+            case REVSW:
+                reverse = 1;
+                continue;
+
 	    case CCSW: 
 	    case DATESW: 
 	    case FROMSW: 
@@ -156,6 +168,10 @@ main (int argc, char **argv)
 	    case NLISTSW: 
 		listsw = 0;
 		continue;
+
+	    case DEBUGSW:
+		++debug;
+		continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
@@ -221,13 +237,22 @@ main (int argc, char **argv)
      * match.  If there is NOT a match, then add it to a list to
      * remove from the final sequence (it will make sense later)
      */
-    for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
+    if (!reverse) { /* Overflow or underflow is fine. */
+        start = mp->lowsel;
+        end = mp->hghsel + 1;
+        inc = 1;
+    } else {
+        start = mp->hghsel;
+        end = mp->lowsel - 1;
+        inc = -1;
+    }
+    for (msgnum = start; msgnum != end; msgnum += inc) {
 	if (is_selected (mp, msgnum)) {
 	    if ((fp = fopen (cp = m_name (msgnum), "r")) == NULL)
 		admonish (cp, "unable to read message");
-	    if (fp && pmatches (fp, msgnum, 0L, 0L)) {
+	    if (fp && pmatches (fp, msgnum, 0L, 0L, debug)) {
 		if (listsw)
-		    printf ("%s\n", m_name (msgnum));
+		    puts(m_name (msgnum));
 	    } else {
 	    	app_msgnum(&nums, msgnum);
 	    }
@@ -280,7 +305,7 @@ main (int argc, char **argv)
      * Print total matched if not printing each matched message number.
      */
     if (!listsw) {
-	printf ("%d hit%s\n", mp2->numsel, mp2->numsel == 1 ? "" : "s");
+	printf ("%d hit%s\n", mp2->numsel, PLURALS(mp2->numsel));
     }
 
     svector_free (seqs);
@@ -294,10 +319,10 @@ main (int argc, char **argv)
 }
 
 
-static void
+static void NORETURN
 putzero_done (int status)
 {
     if (listsw && status && !isatty (fileno (stdout)))
-	printf ("0\n");
+	puts("0");
     exit (status);
 }
