@@ -1,6 +1,4 @@
-
-/*
- * rmf.c -- remove a folder
+/* rmf.c -- remove a folder
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -8,6 +6,9 @@
  */
 
 #include <h/mh.h>
+#include <h/utils.h>
+#include "../sbr/m_maildir.h"
+#include "../sbr/m_mktemp.h"
 
 #define RMF_SWITCHES \
     X("interactive", 0, INTRSW) \
@@ -91,7 +92,7 @@ main (int argc, char **argv)
 	interactive = defolder;
 
     if (strchr (folder, '/') && (*folder != '/') && (*folder != '.')) {
-	for (cp = copy (folder, newfolder); cp > newfolder && *cp != '/'; cp--)
+	for (cp = stpcpy(newfolder, folder); cp > newfolder && *cp != '/'; cp--)
 	    continue;
 	if (cp > newfolder)
 	    *cp = '\0';
@@ -103,7 +104,7 @@ main (int argc, char **argv)
 
     if (interactive) {
 	cp = concat ("Remove folder \"", folder, "\"? ", NULL);
-	if (!getanswer (cp))
+	if (!read_yes_or_no_if_tty (cp))
 	    done (0);
 	free (cp);
     }
@@ -123,16 +124,17 @@ main (int argc, char **argv)
 static int
 rmf (char *folder)
 {
-    int i, j, others;
-    register char *maildir;
+    int i, others;
+    char *maildir;
     char cur[BUFSIZ];
-    register struct dirent *dp;
-    register DIR *dd;
+    struct dirent *dp;
+    DIR *dd;
 
     switch (i = chdir (maildir = m_maildir (folder))) {
 	case OK: 
 	    if (access (".", W_OK) != NOTOK && access ("..", W_OK) != NOTOK)
-		break;		/* fall otherwise */
+		break;
+	    /* FALLTHRU */
 
 	case NOTOK: 
 	    snprintf (cur, sizeof(cur), "atr-%s-%s",
@@ -141,7 +143,7 @@ rmf (char *folder)
 		printf ("[+%s de-referenced]\n", folder);
 		return OK;
 	    }
-	    advise (NULL, "you have no profile entry for the %s folder +%s",
+	    inform("you have no profile entry for the %s folder +%s",
 		    i == NOTOK ? "unreadable" : "read-only", folder);
 	    return NOTOK;
     }
@@ -154,15 +156,15 @@ rmf (char *folder)
      *	Run the external delete hook program.
      */
 
-    (void)ext_hook("del-hook", maildir, (char *)0);
+    (void)ext_hook("del-hook", maildir, NULL);
 
-    j = strlen(BACKUP_PREFIX);
     while ((dp = readdir (dd))) {
 	switch (dp->d_name[0]) {
 	    case '.': 
 		if (strcmp (dp->d_name, ".") == 0
 			|| strcmp (dp->d_name, "..") == 0)
-		    continue;	/* else fall */
+		    continue;
+		break;
 
 	    case ',': 
 		break;
@@ -171,10 +173,10 @@ rmf (char *folder)
 		if (m_atoi (dp->d_name))
 		    break;
 		if (strcmp (dp->d_name, LINK) == 0
-			|| strncmp (dp->d_name, BACKUP_PREFIX, j) == 0)
+			|| has_prefix(dp->d_name, BACKUP_PREFIX))
 		    break;
 
-		admonish (NULL, "file \"%s/%s\" not deleted",
+		inform("file \"%s/%s\" not deleted, continuing...",
 			folder, dp->d_name);
 		others++;
 		continue;
@@ -193,11 +195,13 @@ rmf (char *folder)
      */
     rma (folder);
 
-    chdir ("..");
+    if (chdir ("..") < 0) {
+	advise ("..", "chdir");
+    }
     if (others == 0 && remdir (maildir))
 	return OK;
 
-    advise (NULL, "folder +%s not removed", folder);
+    inform("folder +%s not removed", folder);
     return NOTOK;
 }
 
@@ -210,11 +214,11 @@ rmf (char *folder)
 static void
 rma (char *folder)
 {
-    register int alen, j, plen;
-    register char *cp;
-    register struct node *np, *pp;
+    int alen, j, plen;
+    char *cp;
+    struct node *np, *pp;
 
-    alen = strlen ("atr-");
+    alen = LEN("atr-");
     plen = strlen (cp = m_mailpath (folder)) + 1;
 
     /*
@@ -227,7 +231,8 @@ rma (char *folder)
 		&& *(np->n_name + j) == '-'
 		&& strcmp (cp, np->n_name + j + 1) == 0) {
 	    if (!np->n_context)
-		admonish (NULL, "bug: context_del(key=\"%s\")", np->n_name);
+		inform("bug: context_del(key=\"%s\"), continuing...",
+		    np->n_name);
 	    if (pp) {
 		pp->n_next = np->n_next;
 		np = pp;
