@@ -1,6 +1,4 @@
-
-/*
- * rmm.c -- remove a message(s)
+/* rmm.c -- remove a message(s)
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -9,6 +7,7 @@
 
 #include <h/mh.h>
 #include <h/utils.h>
+#include "../sbr/m_maildir.h"
 
 #define RMM_SWITCHES \
     X("unlink", 0, UNLINKSW) \
