@@ -1,6 +1,4 @@
-
-/*
- * scansbr.c -- routines to help scan along...
+/* scansbr.c -- routines to help scan along...
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -14,17 +12,6 @@
 #include <h/tws.h>
 #include <h/utils.h>
 
-#define MAXSCANL 256		/* longest possible scan line */
-
-/*
- * Buffer size for content part of header fields.  We want this
- * to be large enough so that we don't do a lot of extra FLDPLUS
- * calls on m_getfld but small enough so that we don't snarf
- * the entire message body when we're only going to display 30
- * characters of it.
- */
-#define SBUFSIZ 512
-
 static struct format *fmt;
 static struct comp *datecomp;		/* pntr to "date" comp             */
 static struct comp *bodycomp;		/* pntr to "body" pseudo-comp      *
@@ -35,30 +22,31 @@ static struct comp **used_buf = 0;	/* stack for comp that use buffers */
 
 static int dat[5];			/* aux. data for format routine    */
 
-char *scanl = 0;			/* text of most recent scanline    */
-m_getfld_state_t gstate;		/* for access by msh */
+static m_getfld_state_t gstate;		/* for accessor functions below    */
 
 #define DIEWRERR() adios (scnmsg, "write error on")
 
+#define PUTC(c) \
+    if (putc((c), scnout) == EOF) \
+        DIEWRERR();
+
 #define FPUTS(buf) {\
-		if (mh_fputs(buf,scnout) == EOF)\
+		if (fputs(buf,scnout) == EOF)\
 		    DIEWRERR();\
 		}
 
-/*
- * prototypes
- */
-static int mh_fputs(char *, FILE *);
-
-#ifdef MULTIBYTE_SUPPORT
-#define SCAN_CHARWIDTH MB_CUR_MAX
-#else
-#define SCAN_CHARWIDTH 1
-#endif
+/* outnum determines how the input from inb is copied.  If positive then
+ * it is the number of the message to create, e.g. for inc(1), and all
+ * of the email is copied into that message, with some tweaks.  If 0,
+ * e.g. `scan 42', then reading inb can dubiously stop after a whole
+ * buffer of body, even though this might not be enough to fulfill the
+ * scan format and width.  Or if -1 then no copy is being created, but
+ * all of inb must be read because the next message must be found, e.g.
+ * `scan -file foo.mbox'. */
 
 int
 scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
-      int unseen, char *folder, long size, int noisy)
+      int unseen, char *folder, long size, int noisy, charstring_t *scanl)
 {
     int i, compnum, encrypted, state;
     char *cp, *tmpbuf, *startbody, **nxtbuf;
@@ -70,21 +58,21 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
     char name[NAMESZ];
     int bufsz;
     static int rlwidth, slwidth;
-    static size_t scanl_size;
 
-    /* first-time only initialization */
-    if (!scanl) {
-	if (width == 0) {
+    /* first-time only initialization, which will always happen the
+       way the code is now, with callers initializing *scanl to NULL.
+       scanl used to be a global. */
+    if (! *scanl) {
+	if (width == -1) {
+	    /* Default:  width of the terminal, but at least WIDTH/2. */
 	    if ((width = sc_width ()) < WIDTH/2)
 		width = WIDTH/2;
-	    else if (width > MAXSCANL)
-		width = MAXSCANL;
+	} else if (width == 0) {
+	    /* Unlimited width. */
+	    width = INT_MAX;
 	}
 	dat[3] = slwidth = width;
-	/* Arbitrarily allocate 20 * slwidth to provide room for lots
-	   of escape sequences. */
-	scanl_size = SCAN_CHARWIDTH * (20 * slwidth + 2);
-	scanl = (char *) mh_xmalloc (scanl_size);
+        *scanl = charstring_create (min(width, NMH_BUFSIZ));
 	if (outnum)
 	    umask(~m_gmprot());
 
@@ -95,7 +83,7 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
 	datecomp = fmt_findcomp("date");
 	cptr = fmt_findcomp("folder");
 	if (cptr && folder)
-	    cptr->c_text = getcpy(folder);
+	    cptr->c_text = mh_xstrdup(folder);
 	if (fmt_addcompentry("encrypted")) {
 		ncomps++;
 	}
@@ -135,15 +123,10 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
 	 * and it's our responsibility to free it.
 	 */
 
-	nxtbuf = compbuffers = (char **) calloc((size_t) ncomps, sizeof(char *));
-	if (nxtbuf == NULL)
-	    adios (NULL, "unable to allocate component buffers");
-	used_buf = (struct comp **) calloc((size_t) (ncomps+1),
-	    sizeof(struct comp *));
-	if (used_buf == NULL)
-	    adios (NULL, "unable to allocate component buffer stack");
+	nxtbuf = compbuffers = mh_xcalloc(ncomps, sizeof *nxtbuf);
+	used_buf = mh_xcalloc(ncomps + 1, sizeof *used_buf);
 	used_buf += ncomps+1; *--used_buf = 0;
-	rlwidth = bodycomp && (width > SBUFSIZ) ? width : SBUFSIZ;
+	rlwidth = NMH_BUFSIZ;
 	for (i = ncomps; i--; )
 	    *nxtbuf++ = mh_xmalloc(rlwidth);
     }
@@ -169,21 +152,16 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
 	if (ferror(inb)) {
 	    advise("read", "unable to"); /* "read error" */
 	    return SCNFAT;
-	} else {
-	    return SCNEOF;
 	}
+        return SCNEOF;
     }
 
-    if (outnum) {
-	if (outnum > 0) {
-	    scnmsg = m_name (outnum);
-	    if (*scnmsg == '?')		/* msg num out of range */
-		return SCNNUM;
-	} else {
-	    scnmsg = "/dev/null";
-	}
-	if ((scnout = fopen (scnmsg, "w")) == NULL)
-	    adios (scnmsg, "unable to write");
+    if (outnum > 0) {
+        scnmsg = m_name (outnum);
+        if (*scnmsg == '?')		/* msg num out of range */
+            return SCNNUM;
+        if ((scnout = fopen (scnmsg, "w")) == NULL)
+            adios (scnmsg, "unable to write");
     }
 
     /* scan - main loop */
@@ -193,9 +171,9 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
 	    case FLD: 
 	    case FLDPLUS: 
 		compnum++;
-		if (outnum) {
+		if (scnout) {
 		    FPUTS (name);
-		    if ( putc (':', scnout) == EOF) DIEWRERR();
+		    PUTC(':');
 		    FPUTS (tmpbuf);
 		}
 		/*
@@ -221,13 +199,12 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
 		while (state == FLDPLUS) {
 		    bufsz = rlwidth;
 		    state = m_getfld (&gstate, name, tmpbuf, &bufsz, inb);
-		    if (outnum)
+		    if (scnout)
 			FPUTS (tmpbuf);
 		}
 		break;
 
 	    case BODY: 
-		compnum = -1;
 		/*
 		 * A slight hack ... if we have less than rlwidth characters
 		 * in the buffer, call m_getfld again.
@@ -238,19 +215,21 @@ scan (FILE *inb, int innum, int outnum, char *nfs, int width, int curflg,
 		    state = m_getfld (&gstate, name, tmpbuf + i, &bufsz, inb);
 		}
 
-		if (! outnum) {
+		if (outnum == 0) {
 		    state = FILEEOF; /* stop now if scan cmd */
 		    if (bodycomp && startbody == NULL)
 		    	startbody = tmpbuf;
 		    goto finished;
 		}
-		if (putc ('\n', scnout) == EOF) DIEWRERR();
-		FPUTS (tmpbuf);
+                if (scnout) {
+                    PUTC('\n');
+                    FPUTS (tmpbuf);
+                }
 		/*
                  * The previous code here used to call m_getfld() using
                  * pointers to the underlying output stdio buffers to
                  * avoid the extra copy.  Tests by Markus Schnalke show
-                 * no noticable performance loss on larger mailboxes
+                 * no noticeable performance loss on larger mailboxes
                  * if we incur an extra copy, and messing around with
                  * internal stdio buffers is becoming more and more
                  * unportable as times go on.  So from now on just deal
@@ -271,7 +250,8 @@ body:;
 		while (state == BODY) {
 		    bufsz = rlwidth;
 		    state = m_getfld (&gstate, name, tmpbuf, &bufsz, inb);
-		    FPUTS(tmpbuf);
+                    if (scnout)
+                        FPUTS(tmpbuf);
 		}
 		goto finished;
 
@@ -285,14 +265,14 @@ body:;
 
 		fprintf (stderr, "component %d\n", compnum);
 
-		if (outnum) {
+		if (scnout) {
 		    FPUTS ("\n\nBAD MSG:\n");
 		    FPUTS (name);
-		    if (putc ('\n', scnout) == EOF) DIEWRERR();
+		    PUTC('\n');
 		    state = BODY;
 		    goto body;
 		}
-		/* fall through */
+		goto finished;
 
 	    case FILEEOF:
 		goto finished;
@@ -319,7 +299,7 @@ finished:
 
     if (size)
 	dat[2] = size;
-    else if (outnum > 0)
+    else if (scnout)
     {
 	dat[2] = ftell(scnout);
 	if (dat[2] == EOF) DIEWRERR();
@@ -334,10 +314,7 @@ finished:
 	if (datecomp) {
 	    if (! datecomp->c_text) {
 		if (datecomp->c_tws == NULL)
-		    datecomp->c_tws = (struct tws *)
-			calloc((size_t) 1, sizeof(*datecomp->c_tws));
-		if (datecomp->c_tws == NULL)
-		    adios (NULL, "unable to allocate tws buffer");
+		    NEW0(datecomp->c_tws);
 		*datecomp->c_tws = *dlocaltime ((time_t *) &st.st_mtime);
 		datecomp->c_flags |= CF_DATEFAB|CF_TRUE;
 	    } else {
@@ -346,13 +323,13 @@ finished:
 	}
     }
 
-    fmt_scan (fmt, scanl, scanl_size, slwidth, dat, NULL);
+    fmt_scan (fmt, *scanl, slwidth, dat, NULL);
 
     if (bodycomp)
 	bodycomp->c_text = saved_c_text;
 
     if (noisy)
-	fputs (scanl, stdout);
+	fputs (charstring_buffer (*scanl), stdout);
 
     cptr = fmt_findcomp ("encrypted");
     encrypted = cptr && cptr->c_text;
@@ -362,27 +339,16 @@ finished:
 	cptr->c_text = NULL;
     }
 
-    if (outnum && (ferror(scnout) || fclose (scnout) == EOF))
+    if (scnout && (ferror(scnout) || fclose (scnout) == EOF))
 	DIEWRERR();
 
     return (state != FILEEOF ? SCNERR : encrypted ? SCNENC : SCNMSG);
 }
 
 
-static int
-mh_fputs(char *s, FILE *stream)
-{
-    char c;
-
-    while ((c = *s++)) 
-	if (putc (c,stream) == EOF )
-	    return(EOF);
-    return (0);
-}
-
-/* The following three functions allow access to the global gstate above. */
+/* The following two functions allow access to the global gstate above. */
 void
-scan_finished () {
+scan_finished(void) {
     m_getfld_state_destroy (&gstate);
 }
 
@@ -390,13 +356,3 @@ void
 scan_detect_mbox_style (FILE *f) {
     m_unknown (&gstate, f);
 }
-
-void
-scan_eom_action (int (*action)()) {
-    m_eomsbr (gstate, action);
-}
-
-void
-scan_reset_m_getfld_state () {
-    m_getfld_state_reset (&gstate);
-}
