@@ -1,6 +1,4 @@
-
-/*
- * sendsbr.c -- routines to help WhatNow/Send along
+/* sendsbr.c -- routines to help WhatNow/Send along
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -8,6 +6,8 @@
  */
 
 #include <h/mh.h>
+#include <h/fmt_scan.h>
+#include <h/fmt_compile.h>
 #include <h/signals.h>
 #include <setjmp.h>
 #include <fcntl.h>
@@ -21,6 +21,17 @@
 #endif
 #include <time.h>
 
+#ifdef OAUTH_SUPPORT
+#include <h/oauth.h>
+#endif
+#include "../sbr/m_maildir.h"
+#include "../sbr/m_mktemp.h"
+#include "../sbr/message_id.h"
+
+#ifdef OAUTH_SUPPORT
+static int setup_oauth_params(char *[], int *, const char *, const char **);
+#endif /* OAUTH_SUPPORT */
+
 int debugsw = 0;		/* global */
 int forwsw  = 1;
 int inplace = 1;
@@ -38,14 +49,17 @@ static jmp_buf env;
 /*
  * static prototypes
  */
-static void armed_done (int) NORETURN;
 static void alert (char *, int);
 static int tmp_fd (void);
 static void anno (int, struct stat *);
 static void annoaux (int);
 static int splitmsg (char **, int, char *, char *, struct stat *, int);
 static int sendaux (char **, int, char *, char *, struct stat *);
-
+static void handle_sendfrom(char **, int *, char *, const char *);
+static int get_from_header_info(const char *, const char **, const char **, const char **);
+static const char *get_message_header_info(FILE *, char *);
+static void merge_profile_entry(const char *, const char *, char *[], int *);
+static void armed_done (int) NORETURN;
 
 /*
  * Entry point into (back-end) routines to send message.
@@ -53,7 +67,7 @@ static int sendaux (char **, int, char *, char *, struct stat *);
 
 int
 sendsbr (char **vec, int vecp, char *program, char *draft, struct stat *st,
-         int rename_drft)
+         int rename_drft, const char *auth_svc)
 {
     int status, i;
     pid_t child;
@@ -61,6 +75,10 @@ sendsbr (char **vec, int vecp, char *program, char *draft, struct stat *st,
     struct stat sts;
     char **buildvec, *buildprogram;
     char *volatile drft = draft;
+    /* nvecs is volatile to prevent warning from gcc about possible clobbering
+       by longjmp. */
+    volatile int nvecs = vecp;
+    int *nvecsp = (int *) &nvecs;
 
     /*
      * Run the mimebuildproc (which is by default mhbuild) on the message
@@ -73,7 +91,7 @@ sendsbr (char **vec, int vecp, char *program, char *draft, struct stat *st,
 	break;
 
     case OK:
-	buildvec = argsplit(buildmimeproc, &buildprogram, &i);
+        buildvec = argsplit(buildmimeproc, &buildprogram, &i);
 	buildvec[i++] = "-auto";
 	if (distfile)
 	    buildvec[i++] = "-dist";
@@ -93,7 +111,7 @@ sendsbr (char **vec, int vecp, char *program, char *draft, struct stat *st,
 
     done=armed_done;
     switch (setjmp (env)) {
-    case OK: 
+    case OK:
 	/*
 	 * If given -push and -unique (which is undocumented), then
 	 * rename the draft file.  I'm not quite sure why.
@@ -109,16 +127,40 @@ sendsbr (char **vec, int vecp, char *program, char *draft, struct stat *st,
 	    drft = file;
 	}
 
+	/*
+	 * Add in any necessary profile entries for xoauth
+	 */
+
+	if (auth_svc) {
+#ifdef OAUTH_SUPPORT
+		const char *errmsg;
+		if (setup_oauth_params(vec, nvecsp, auth_svc, &errmsg) != OK) {
+			adios(NULL, errmsg);
+		}
+#else
+                adios(NULL, "send built without OAUTH_SUPPORT, "
+                      "so auth_svc %s is not supported", auth_svc);
+#endif /* OAUTH_SUPPORT */
+	}
+
+        /*
+         * Rework the vec based on From: header in draft, as specified
+         * by sendfrom-address entries in profile.
+         */
+        if (context_find_prefix("sendfrom-")) {
+            handle_sendfrom(vec, nvecsp, draft, auth_svc);
+        }
+
 	/*
 	 * Check if we need to split the message into
 	 * multiple messages of type "message/partial".
 	 */
 	if (splitsw >= 0 && !distfile && stat ((char *) drft, &sts) != NOTOK
 		&& sts.st_size >= CPERMSG) {
-	    status = splitmsg (vec, vecp, program, drft,
+	    status = splitmsg (vec, nvecs, program, drft,
 	    		       st, splitsw) ? NOTOK : OK;
 	} else {
-	    status = sendaux (vec, vecp, program, drft, st) ? NOTOK : OK;
+	    status = sendaux (vec, nvecs, program, drft, st) ? NOTOK : OK;
 	}
 
 	/* rename the original draft */
@@ -128,7 +170,7 @@ sendsbr (char **vec, int vecp, char *program, char *draft, struct stat *st,
 	    advise (buffer, "unable to rename %s to", drft);
 	break;
 
-    default: 
+    default:
 	status = DONE;
 	break;
     }
@@ -152,7 +194,7 @@ splitmsg (char **vec, int vecp, char *program, char *drft,
     int	compnum, nparts, partno, state, status;
     long pos, start;
     time_t clock;
-    char *cp, *dp, buffer[BUFSIZ], msgid[BUFSIZ];
+    char *cp, *dp, buffer[NMH_BUFSIZ], msgid[BUFSIZ];
     char subject[BUFSIZ];
     char name[NAMESZ], partnum[BUFSIZ];
     FILE *in;
@@ -194,7 +236,7 @@ splitmsg (char **vec, int vecp, char *program, char *drft,
 		     * of message/partials.  For the "Subject" header
 		     * field, we also record it, so that a modified
 		     * version of it, can be copied to the header
-		     * of each messsage/partial in the collection.
+		     * of each message/partial in the collection.
 		     */
 		    if (!strcasecmp (name, "Subject")) {
 			size_t sublen;
@@ -247,7 +289,7 @@ splitmsg (char **vec, int vecp, char *program, char *drft,
 
     nparts = 1;
     pos = start;
-    while (fgets (buffer, sizeof(buffer) - 1, in)) {
+    while (fgets (buffer, sizeof buffer, in)) {
 	long len;
 
 	if ((pos += (len = strlen (buffer))) > CPERMSG) {
@@ -259,8 +301,7 @@ splitmsg (char **vec, int vecp, char *program, char *drft,
     /* Only one part, nothing to split */
     if (nparts == 1) {
 	free (cp);
-	if (dp)
-	    free (dp);
+        mh_xfree(dp);
 
 	fclose (in);
 	return sendaux (vec, vecp, program, drft, st);
@@ -318,12 +359,12 @@ splitmsg (char **vec, int vecp, char *program, char *drft,
 	for (;;) {
 	    long len;
 
-	    if (!fgets (buffer, sizeof(buffer) - 1, in)) {
+	    if (!fgets (buffer, sizeof buffer, in)) {
 		if (partno == nparts)
 		    break;
 		adios (NULL, "premature eof");
 	    }
-	    
+
 	    if ((pos += (len = strlen (buffer))) > CPERMSG) {
 		fseek (in, -len, SEEK_CUR);
 		break;
@@ -338,7 +379,7 @@ splitmsg (char **vec, int vecp, char *program, char *drft,
 	fclose (out);
 
 	if (!pushsw && verbsw) {
-	    printf ("\n");
+	    putchar('\n');
 	    fflush (stdout);
 	}
 
@@ -366,8 +407,7 @@ splitmsg (char **vec, int vecp, char *program, char *drft,
     }
 
     free (cp);
-    if (dp)
-	free (dp);
+    mh_xfree(dp);
 
     fclose (in);	/* close the draft */
     return status;
@@ -389,17 +429,17 @@ sendaux (char **vec, int vecp, char *program, char *drft, struct stat *st)
     fd = pushsw ? tmp_fd () : NOTOK;
     fd2 = NOTOK;
 
-    vec[vecp++] = drft;
     if (annotext) {
 	if ((fd2 = tmp_fd ()) != NOTOK) {
 	    vec[vecp++] = "-idanno";
 	    snprintf (buf, sizeof(buf), "%d", fd2);
 	    vec[vecp++] = buf;
 	} else {
-	    admonish (NULL, "unable to create temporary file in %s "
-                      "for annotation list", get_temp_dir());
+	    inform("unable to create temporary file in %s for "
+		"annotation list, continuing...", get_temp_dir());
 	}
     }
+    vec[vecp++] = drft;
     if (distfile && distout (drft, distfile, backup) == NOTOK)
 	done (1);
     vec[vecp] = NULL;
@@ -417,7 +457,7 @@ sendaux (char **vec, int vecp, char *program, char *drft, struct stat *st)
 	/*
 	 * child process -- send it
 	 *
-	 * If fd is ok, then we are pushing and fd points to temp
+	 * If fd is OK, then we are pushing and fd points to temp
 	 * file, so capture anything on stdout and stderr there.
 	 */
 	if (fd != NOTOK) {
@@ -447,7 +487,7 @@ sendaux (char **vec, int vecp, char *program, char *drft, struct stat *st)
 		alert (drft, fd);
 		close (fd);
 	    } else {
-		advise (NULL, "message not delivered to anyone");
+		inform("message not delivered to anyone");
 	    }
 	    if (annotext && fd2 != NOTOK)
 		close (fd2);
@@ -485,6 +525,7 @@ alert (char *file, int out)
 	case NOTOK:
 	    /* oops -- fork error */
 	    advise ("fork", "unable to");
+	    /* FALLTHRU */
 
 	case OK:
 	    /* child process -- send it */
@@ -496,20 +537,26 @@ alert (char *file, int out)
 		if ((in = open (file, O_RDONLY)) == NOTOK) {
 		    admonish (file, "unable to re-open");
 		} else {
-		    lseek (out, (off_t) 0, SEEK_END);
+		    lseek(out, 0, SEEK_END);
 		    strncpy (buf, "\nMessage not delivered to anyone.\n", sizeof(buf));
-		    write (out, buf, strlen (buf));
+		    if (write (out, buf, strlen (buf)) < 0) {
+			advise (file, "write");
+		    }
 		    strncpy (buf, "\n------- Unsent Draft\n\n", sizeof(buf));
-		    write (out, buf, strlen (buf));
+		    if (write (out, buf, strlen (buf)) < 0) {
+			advise (file, "write");
+		    }
 		    cpydgst (in, out, file, "temporary file");
 		    close (in);
 		    strncpy (buf, "\n------- End of Unsent Draft\n", sizeof(buf));
-		    write (out, buf, strlen (buf));
+		    if (write (out, buf, strlen (buf)) < 0) {
+			advise (file, "write");
+		    }
 		    if (rename (file, strncpy (buf, m_backup (file), sizeof(buf))) == NOTOK)
 			admonish (buf, "unable to rename %s to", file);
 		}
 	    }
-	    lseek (out, (off_t) 0, SEEK_SET);
+	    lseek(out, 0, SEEK_SET);
 	    dup2 (out, fileno (stdin));
 	    close (out);
 	    /* create subject for error notification */
@@ -543,7 +590,7 @@ tmp_fd (void)
     if ((tfile = m_mktemp2(NULL, invo_name, &fd, NULL)) == NULL) return NOTOK;
 
     if (debugsw)
-	advise (NULL, "temporary file %s selected", tfile);
+	inform("temporary file %s selected", tfile);
     else
 	if (m_unlink (tfile) == NOTOK)
 	    advise (tfile, "unable to remove");
@@ -566,7 +613,7 @@ anno (int fd, struct stat *st)
 		|| st->st_dev != st2.st_dev
 		|| st->st_ino != st2.st_ino)) {
 	if (debugsw)
-	    admonish (NULL, "$mhaltmsg mismatch");
+	    inform("$mhaltmsg mismatch, continuing...");
 	return;
     }
 
@@ -574,12 +621,12 @@ anno (int fd, struct stat *st)
     switch (child_id) {
 	case NOTOK: 		/* oops */
 	    if (!debugsw)
-		advise (NULL,
-			    "unable to fork, so doing annotations by hand...");
+		inform("unable to fork, so doing annotations by hand...");
 	    if (cwd == NULL)
-		cwd = getcpy (pwd ());
+		cwd = mh_xstrdup(pwd ());
+	    /* FALLTHRU */
 
-	case OK: 
+	case OK:
 	    /* block a few signals */
 	    sigemptyset (&set);
 	    sigaddset (&set, SIGHUP);
@@ -597,7 +644,9 @@ anno (int fd, struct stat *st)
 	    /* reset the signal mask */
 	    sigprocmask (SIG_SETMASK, &oset, &set);
 
-	    chdir (cwd);
+	    if (chdir (cwd) < 0) {
+		advise (cwd, "chdir");
+	    }
 	    break;
 
 	default: 		/* no waiting... */
@@ -618,7 +667,7 @@ annoaux (int fd)
 
     if ((folder = getenv ("mhfolder")) == NULL || *folder == 0) {
 	if (debugsw)
-	    admonish (NULL, "$mhfolder not set");
+	    inform("$mhfolder not set, continuing...");
 	return;
     }
     maildir = m_maildir (folder);
@@ -629,20 +678,20 @@ annoaux (int fd)
     }
     if (!(mp = folder_read (folder, 0))) {
 	if (debugsw)
-	    admonish (NULL, "unable to read folder %s", folder);
+	    inform("unable to read folder %s, continuing...", folder);
 	return;
     }
 
     /* check for empty folder */
     if (mp->nummsg == 0) {
 	if (debugsw)
-	    admonish (NULL, "no messages in %s", folder);
+	    inform("no messages in %s, continuing...", folder);
 	goto oops;
     }
 
     if ((cp = getenv ("mhmessages")) == NULL || *cp == 0) {
 	if (debugsw)
-	    admonish (NULL, "$mhmessages not set");
+	    inform("$mhmessages not set, continuing...");
 	goto oops;
     }
     if (!debugsw			/* MOBY HACK... */
@@ -654,21 +703,21 @@ annoaux (int fd)
     }
     else
 	fd2 = NOTOK;
-    for (ap = brkstring (cp = getcpy (cp), " ", NULL); *ap; ap++)
+    for (ap = brkstring (cp = mh_xstrdup(cp), " ", NULL); *ap; ap++)
 	m_convert (mp, *ap);
     free (cp);
     if (fd2 != NOTOK)
 	dup2 (fd2, fileno (stderr));
     if (mp->numsel == 0) {
 	if (debugsw)
-	    admonish (NULL, "no messages to annotate");
+	    inform("no messages to annotate, continuing...");
 	goto oops;
     }
 
-    lseek (fd, (off_t) 0, SEEK_SET);
+    lseek(fd, 0, SEEK_SET);
     if ((fp = fdopen (fd, "r")) == NULL) {
 	if (debugsw)
-	    admonish (NULL, "unable to fdopen annotation list");
+	    inform("unable to fdopen annotation list, continuing...");
 	goto oops;
     }
     cp = NULL;
@@ -677,12 +726,12 @@ annoaux (int fd)
     fclose (fp);
 
     if (debugsw)
-	advise (NULL, "annotate%s with %s: \"%s\"",
+	inform("annotate%s with %s: \"%s\"",
 		inplace ? " inplace" : "", annotext, cp);
     for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
 	if (is_selected(mp, msgnum)) {
 	    if (debugsw)
-		advise (NULL, "annotate message %d", msgnum);
+		inform("annotate message %d", msgnum);
             annotate (m_name (msgnum), annotext, cp, inplace, 1, -2, 0);
 	}
     }
@@ -694,7 +743,277 @@ oops:
 }
 
 
-static void
+static
+void
+handle_sendfrom(char **vec, int *vecp, char *draft, const char *auth_svc) {
+    const char *addr, *host;
+    const char *message;
+
+    /* Extract address and host from From: header line in draft. */
+    if (get_from_header_info(draft, &addr, &host, &message) != OK) {
+        adios(draft, message);
+    }
+
+    /* Merge in any address or host specific switches to post(1) from profile. */
+    merge_profile_entry(addr, host, vec, vecp);
+    free((void *) host);
+    free((void *) addr);
+
+    vec[*vecp] = NULL;
+
+    {
+        char **vp;
+
+        for (vp = vec; *vp; ++vp) {
+            if (strcmp(*vp, "xoauth2") == 0) {
+#ifdef OAUTH_SUPPORT
+                if (setup_oauth_params(vec, vecp, auth_svc, &message) != OK) {
+                    adios(NULL, message);
+                }
+                break;
+#else
+                NMH_UNUSED(auth_svc);
+                adios(NULL, "send built without OAUTH_SUPPORT, "
+                      "so -saslmech xoauth2 is not supported");
+#endif /* OAUTH_SUPPORT */
+            }
+        }
+    }
+}
+
+
+#ifdef OAUTH_SUPPORT
+/*
+ * For XOAUTH2, append profile entries so post can do the heavy lifting
+ */
+static int
+setup_oauth_params(char *vec[], int *vecp, const char *auth_svc,
+		   const char **message) {
+    const char *saslmech = NULL, *user = NULL;
+    mh_oauth_service_info svc;
+    char errbuf[256];
+    int i;
+
+    /* Make sure we have all the information we need. */
+    for (i = 1; i < *vecp; ++i) {
+        /* Don't support abbreviated switches, to avoid collisions in the
+           future if new ones are added. */
+        if (! strcmp(vec[i-1], "-saslmech")) {
+            saslmech = vec[i];
+        } else if (! strcmp(vec[i-1], "-user")) {
+            user = vec[i];
+        } else if (! strcmp(vec[i-1], "-authservice")) {
+            auth_svc = vec[i];
+        }
+    }
+
+    if (auth_svc == NULL) {
+        if (saslmech  &&  ! strcasecmp(saslmech, "xoauth2")) {
+            *message = "must specify -authservice with -saslmech xoauth2";
+            return NOTOK;
+        }
+    } else {
+        if (user == NULL) {
+            *message = "must specify -user with -saslmech xoauth2";
+            return NOTOK;
+        }
+
+        if (saslmech  &&  ! strcasecmp(saslmech, "xoauth2")) {
+	    if (! mh_oauth_get_service_info(auth_svc, &svc, errbuf,
+	    				    sizeof(errbuf)))
+		adios(NULL, "Unable to retrieve oauth profile entries: %s",
+		      errbuf);
+
+	    vec[(*vecp)++] = mh_xstrdup("-authservice");
+	    vec[(*vecp)++] = mh_xstrdup(auth_svc);
+	    vec[(*vecp)++] = mh_xstrdup("-oauthcredfile");
+	    vec[(*vecp)++] = mh_xstrdup(mh_oauth_cred_fn(auth_svc));
+	    vec[(*vecp)++] = mh_xstrdup("-oauthclientid");
+	    vec[(*vecp)++] = getcpy(svc.client_id);
+	    vec[(*vecp)++] = mh_xstrdup("-oauthclientsecret");
+	    vec[(*vecp)++] = getcpy(svc.client_secret);
+	    vec[(*vecp)++] = mh_xstrdup("-oauthauthendpoint");
+	    vec[(*vecp)++] = getcpy(svc.auth_endpoint);
+	    vec[(*vecp)++] = mh_xstrdup("-oauthredirect");
+	    vec[(*vecp)++] = getcpy(svc.redirect_uri);
+	    vec[(*vecp)++] = mh_xstrdup("-oauthtokenendpoint");
+	    vec[(*vecp)++] = getcpy(svc.token_endpoint);
+	    vec[(*vecp)++] = mh_xstrdup("-oauthscope");
+	    vec[(*vecp)++] = getcpy(svc.scope);
+        }
+    }
+
+    return 0;
+}
+#endif /* OAUTH_SUPPORT */
+
+
+/*
+ * Extract user and domain from From: header line in draft.
+ */
+static
+int
+get_from_header_info(const char *filename, const char **addr, const char **host, const char **message) {
+    struct stat st;
+    FILE *in;
+
+    if (stat (filename, &st) == NOTOK) {
+        *message = "unable to stat draft file";
+        return NOTOK;
+    }
+
+    if ((in = fopen (filename, "r")) != NULL) {
+        /* There must be a non-blank Envelope-From or {Resent-}Sender or
+           {Resent-}From header. */
+        char *addrformat = "%(addr{Envelope-From})";
+        char *hostformat = "%(host{Envelope-From})";
+
+        if ((*addr = get_message_header_info (in, addrformat)) == NULL  ||
+            !**addr) {
+            addrformat = distfile == NULL  ?  "%(addr{Sender})"  :  "%(addr{Resent-Sender})";
+            hostformat = distfile == NULL  ?  "%(host{Sender})"  :  "%(host{Resent-Sender})";
+
+            if ((*addr = get_message_header_info (in, addrformat)) == NULL) {
+                addrformat = distfile == NULL  ?  "%(addr{From})"  :  "%(addr{Resent-From})";
+                hostformat = distfile == NULL  ?  "%(host{From})"  :  "%(host{Resent-From})";
+
+                if ((*addr = get_message_header_info (in, addrformat)) == NULL) {
+                    *message = "unable to find sender address in";
+                    fclose(in);
+                    return NOTOK;
+                }
+            }
+        }
+
+        /* Use the hostformat that corresponds to the successful addrformat. */
+        if ((*host = get_message_header_info(in, hostformat)) == NULL) {
+            *message = "unable to find sender host";
+            fclose(in);
+            return NOTOK;
+        }
+        fclose(in);
+
+        return OK;
+    }
+
+    *message = "unable to open";
+    return NOTOK;
+}
+
+
+/*
+ * Get formatted information from header of a message.
+ * Adapted from process_single_file() in uip/fmttest.c.
+ */
+static
+const char *
+get_message_header_info(FILE *in, char *format) {
+    int dat[5];
+    struct format *fmt;
+    struct stat st;
+    int parsing_header;
+    m_getfld_state_t gstate = 0;
+    charstring_t buffer = charstring_create(0);
+    char *retval;
+
+    dat[0] = dat[1] = dat[4] = 0;
+    dat[2] = fstat(fileno(in), &st) == 0  ?  st.st_size  :  0;
+    dat[3] = INT_MAX;
+
+    (void) fmt_compile(new_fs(NULL, NULL, format), &fmt, 1);
+    free_fs();
+
+    /*
+     * Read in the message and process the header.
+     */
+    rewind (in);
+    parsing_header = 1;
+    do {
+        char name[NAMESZ], rbuf[NMH_BUFSIZ];
+        int bufsz = sizeof rbuf;
+        int state = m_getfld(&gstate, name, rbuf, &bufsz, in);
+
+        switch (state) {
+        case FLD:
+        case FLDPLUS: {
+            int bucket = fmt_addcomptext(name, rbuf);
+
+            if (bucket != -1) {
+                while (state == FLDPLUS) {
+                    bufsz = sizeof rbuf;
+                    state = m_getfld(&gstate, name, rbuf, &bufsz, in);
+                    fmt_appendcomp(bucket, name, rbuf);
+                }
+            }
+
+            while (state == FLDPLUS) {
+                bufsz = sizeof rbuf;
+                state = m_getfld(&gstate, name, rbuf, &bufsz, in);
+            }
+            break;
+        }
+        default:
+            parsing_header = 0;
+        }
+    } while (parsing_header);
+    m_getfld_state_destroy(&gstate);
+
+    fmt_scan(fmt, buffer, INT_MAX, dat, NULL);
+    fmt_free(fmt, 1);
+
+    /* Trim trailing newline, if any. */
+    retval = rtrim(charstring_buffer_copy((buffer)));
+    charstring_free(buffer);
+    if (*retval)
+        return retval;
+
+    free(retval);
+    return NULL;
+}
+
+
+/*
+ * Look in profile for entry corresponding to addr or host, and add its contents to vec.
+ *
+ * Could do some of this automatically, by looking for:
+ * 1) access-$(mbox{from}) in oauth-svc file using mh_oauth_cred_load(), which isn't
+ *    static and doesn't have side effects; free the result with mh_oauth_cred_free())
+ * 2) machine $(mbox{from}) in creds
+ * If no -server passed in from profile or commandline, could use smtp.<svc>.com for gmail,
+ * but that might not generalize for other svcs.
+ */
+static
+void
+merge_profile_entry(const char *addr, const char *host, char *vec[], int *vecp) {
+    char *addr_entry = concat("sendfrom-", addr, NULL);
+    char *profile_entry = context_find(addr_entry);
+
+    free(addr_entry);
+    if (profile_entry == NULL) {
+        /* No entry for the user.  Look for one for the host. */
+        char *host_entry = concat("sendfrom-", host, NULL);
+
+        profile_entry = context_find(host_entry);
+        free(host_entry);
+    }
+
+    /* Use argsplit() to do the real work of splitting the args in the profile entry. */
+    if (profile_entry && *profile_entry) {
+        int profile_vecp;
+        char *file;
+        char **profile_vec = argsplit(profile_entry, &file, &profile_vecp);
+        int i;
+
+        for (i = 0; i < profile_vecp; ++i) {
+            vec[(*vecp)++] = getcpy(profile_vec[i]);
+        }
+
+        arglist_free(file, profile_vec);
+    }
+}
+
+
+static void NORETURN
 armed_done (int status)
 {
     longjmp (env, status ? status : NOTOK);
