@@ -1,6 +1,4 @@
-
-/*
- * slocal.c -- asynchronously filter and deliver new mail
+/* slocal.c -- asynchronously filter and deliver new mail
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -29,6 +27,8 @@
 #include <h/tws.h>
 #include <h/mts.h>
 #include <h/utils.h>
+#include "../sbr/lock_file.h"
+#include "../sbr/m_mktemp.h"
 
 #include <pwd.h>
 #include <sys/ioctl.h>
@@ -190,7 +190,7 @@ main (int argc, char **argv)
 
     if (nmh_init(argv[0], 0 /* use context_foil() */)) { return 1; }
 
-    mts_init (invo_name);
+    mts_init ();
     arguments = getarguments (invo_name, argc, argv, 0);
     argp = arguments;
 
@@ -273,13 +273,16 @@ main (int argc, char **argv)
 
     if (addr == NULL)
 	addr = getusername ();
-    if (user == NULL)
-	user = (cp = strchr(addr, '.')) ? ++cp : addr;
+    if (user == NULL) {
+	user = getusername ();
+    }
     if ((pw = getpwnam (user)) == NULL)
 	adios (NULL, "no such local user as %s", user);
 
     if (chdir (pw->pw_dir) == -1)
-	chdir ("/");
+	if (chdir ("/") < 0) {
+	    advise ("/", "chdir");
+	}
     umask (0077);
 
     if (geteuid() == 0) {
@@ -420,7 +423,7 @@ usr_delivery (int fd, char *delivery, int su)
     int i, accept, status=1, won, vecp, next;
     char *field, *pattern, *action, *result, *string;
     char buffer[BUFSIZ], tmpbuf[BUFSIZ];
-    char *cp, *vec[NVEC];
+    char *vec[NVEC];
     struct stat st;
     struct pair *p;
     FILE *fp;
@@ -449,9 +452,7 @@ usr_delivery (int fd, char *delivery, int su)
 	if (*buffer == '#' || *buffer == '\n')
 	    continue;
 
-	/* zap trailing newline */
-	if ((cp = strchr(buffer, '\n')))
-	    *cp = 0;
+        trim_suffix_c(buffer, '\n');
 
 	/* split buffer into fields */
 	vecp = split (buffer, vec);
@@ -483,7 +484,8 @@ usr_delivery (int fd, char *delivery, int su)
 		 * do this - else fall through
 		 */
  		if (!next)
-		    continue;	/* else fall */
+		    continue;
+		/* FALLTHRU */
 
 	    case '?': 
 		/*
@@ -491,7 +493,8 @@ usr_delivery (int fd, char *delivery, int su)
 		 * consider delivered if action is successful.
 		 */
 		if (won)
-		    continue;	/* else fall */
+		    continue;
+		/* FALLTHRU */
 
 	    case 'A': 
 	    case 'a': 
@@ -537,7 +540,8 @@ usr_delivery (int fd, char *delivery, int su)
 		    if (won)
 			continue;
 		    break;
-		}		/* else fall */
+		}
+		/* FALLTHRU */
 
 	    default: 
 		/* parse message and build lookup table */
@@ -564,7 +568,8 @@ usr_delivery (int fd, char *delivery, int su)
 	    case 'q':
 		/* deliver to quoted pipe */
 		if (strcasecmp (action, "qpipe"))
-		    continue;	/* else fall */
+		    continue;
+		/* FALLTHRU */
 	    case '^':
 		expand (tmpbuf, string, fd);
 		if (split (tmpbuf, vec) < 1)
@@ -575,7 +580,8 @@ usr_delivery (int fd, char *delivery, int su)
 	    case 'p': 
 		/* deliver to pipe */
 		if (strcasecmp (action, "pipe"))
-		    continue;	/* else fall */
+		    continue;
+		/* FALLTHRU */
 	    case '|': 
 		vec[2] = "sh";
 		vec[3] = "-c";
@@ -593,7 +599,8 @@ usr_delivery (int fd, char *delivery, int su)
 		}
 		/* deliver to nmh folder */
 		else if (strcasecmp (action, "folder"))
-		    continue;	/* else fall */
+		    continue;
+		/* FALLTHRU */
 	    case '+':
 		status = usr_folder (fd, string);
 		break;
@@ -606,7 +613,8 @@ usr_delivery (int fd, char *delivery, int su)
 		}
 		/* mbox format */
 		else if (strcasecmp (action, "mbox"))
-		    continue;	/* else fall */
+		    continue;
+		/* FALLTHRU */
 
 	    case '>': 
 		/* mbox format */
@@ -702,7 +710,7 @@ parse (int fd)
     int i, state;
     int fd1;
     char *cp, *dp, *lp;
-    char name[NAMESZ], field[BUFSIZ];
+    char name[NAMESZ], field[NMH_BUFSIZ];
     struct pair *p, *q;
     FILE  *in;
     m_getfld_state_t gstate = 0;
@@ -734,7 +742,7 @@ parse (int fd)
 	switch (state = m_getfld (&gstate, name, field, &fieldsz, in)) {
 	    case FLD: 
 	    case FLDPLUS: 
-		lp = add (field, NULL);
+		lp = mh_xstrdup(field);
 		while (state == FLDPLUS) {
 		    fieldsz = sizeof field;
 		    state = m_getfld (&gstate, name, field, &fieldsz, in);
@@ -760,7 +768,7 @@ parse (int fd)
 		    }
 		}
 		if (p->p_name == NULL && i < NVEC) {
-		    p->p_name = getcpy (name);
+		    p->p_name = mh_xstrdup(name);
 		    p->p_value = lp;
 		    p->p_flags = P_NIL;
 		    p++, i++;
@@ -774,11 +782,11 @@ parse (int fd)
 
 	    case LENERR: 
 	    case FMTERR: 
-		advise (NULL, "format error in message");
+		inform("format error in message");
 		break;
 
 	    default: 
-		advise (NULL, "internal error in m_getfld");
+		inform("internal error in m_getfld");
 		fclose (in);
 		return -1;
 	}
@@ -871,7 +879,7 @@ glob (int fd)
     if ((p = lookup (vars, "size"))) {
 	snprintf (buffer, sizeof(buffer), "%d",
 		fstat (fd, &st) != -1 ? (int) st.st_size : 0);
-	p->p_value = getcpy (buffer);
+	p->p_value = mh_xstrdup(buffer);
     }
     if ((p = lookup (vars, "info")))
 	p->p_value = getcpy (info);
@@ -963,19 +971,11 @@ timely (char *t1, char *t2)
 static int
 usr_file (int fd, char *mailbox, int mbx_style)
 {
-    int	md, mapping;
+    int	md;
 
-    if (verbose)
-	verbose_printf ("delivering to file \"%s\"", mailbox);
-
-    if (mbx_style == MBOX_FORMAT) {
-	if (verbose)
-	    verbose_printf (" (mbox style)");
-	mapping = 0;
-    } else {
-	if (verbose)
-	    verbose_printf (" (mmdf style)");
-	mapping = 1;
+    if (verbose) {
+        verbose_printf("delivering to file \"%s\" (%s style)", mailbox,
+            mbx_style == MBOX_FORMAT ? "mbox" : "mmdf");
     }
 
     /* open and lock the file */
@@ -985,10 +985,10 @@ usr_file (int fd, char *mailbox, int mbx_style)
 	return -1;
     }
 
-    lseek (fd, (off_t) 0, SEEK_SET);
+    lseek(fd, 0, SEEK_SET);
 
     /* append message to file */
-    if (mbx_copy (mailbox, mbx_style, md, fd, mapping, NULL, verbose) == -1) {
+    if (mbx_copy (mailbox, mbx_style, md, fd, NULL) == -1) {
 	if (verbose)
 	    adorn ("", "error writing to:");
 	return -1;
@@ -1051,7 +1051,7 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
     if (verbose && !suppress)
 	verbose_printf ("delivering to pipe \"%s\"", cmd);
 
-    lseek (fd, (off_t) 0, SEEK_SET);
+    lseek(fd, 0, SEEK_SET);
 
     for (i = 0; (child_id = fork()) == -1 && i < 5; i++)
 	sleep (5);
@@ -1067,8 +1067,12 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 	    /* child process */
 	    if (fd != 0)
 		dup2 (fd, 0);
-	    freopen ("/dev/null", "w", stdout);
-	    freopen ("/dev/null", "w", stderr);
+	    if (freopen ("/dev/null", "w", stdout) == NULL) {
+		advise ("stdout", "freopen");
+	    }
+	    if (freopen ("/dev/null", "w", stderr) == NULL) {
+		advise ("stderr", "freopen");
+	    }
 	    if (fd != 3)
 		dup2 (fd, 3);
 	    closefds (4);
@@ -1083,9 +1087,9 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 	    setpgid ((pid_t) 0, getpid ());	/* put in own process group */
 
 	    *environ = NULL;
-	    m_putenv ("USER", pw->pw_name);
-	    m_putenv ("HOME", pw->pw_dir);
-	    m_putenv ("SHELL", pw->pw_shell);
+	    setenv("USER", pw->pw_name, 1);
+	    setenv("HOME", pw->pw_dir, 1);
+	    setenv("SHELL", pw->pw_shell, 1);
 
 	    execvp (pgm, vec);
 	    _exit (-1);
@@ -1120,16 +1124,15 @@ usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 			    pidstatus (status, stdout, ", failed");
 		}
 		return (status == 0 ? 0 : -1);
-	    } else {
-		/*
-		 * Ruthlessly kill the child and anything
-		 * else in its process group.
-		 */
-		killpg(child_id, SIGKILL);
-		if (verbose)
-		    verbose_printf (", timed-out; terminated\n");
-		return -1;
 	    }
+            /*
+             * Ruthlessly kill the child and anything
+             * else in its process group.
+             */
+            killpg(child_id, SIGKILL);
+            if (verbose)
+                verbose_printf (", timed-out; terminated\n");
+            return -1;
     }
 }
 
@@ -1156,11 +1159,11 @@ get_sender (char *envelope, char **sender)
     char buffer[BUFSIZ];
 
     if (envelope == NULL) {
-	*sender = getcpy ("");
+	*sender = mh_xstrdup("");
 	return;
     }
 
-    i = strlen ("From ");
+    i = LEN("From ");
     strncpy (buffer, envelope + i, sizeof(buffer));
     if ((cp = strchr(buffer, '\n'))) {
 	*cp = 0;
@@ -1177,7 +1180,7 @@ get_sender (char *envelope, char **sender)
 	    *cp = 0;
 	else
 	    break;
-    *sender = getcpy (buffer);
+    *sender = mh_xstrdup(buffer);
 }
 
 
@@ -1209,7 +1212,7 @@ you_lose:
 	    }
 	if (i == -1)
 	    goto you_lose;
-	lseek (fd1, (off_t) 0, SEEK_SET);
+	lseek(fd1, 0, SEEK_SET);
 	return fd1;
     }
 
@@ -1246,13 +1249,12 @@ you_lose:
      * and massage the headers.  Save
      * a copy of the "From " line for later.
      */
-    i = strlen ("From ");
     while (fgets (buffer, sizeof(buffer), qfp)) {
 	if (first) {
 	    first = 0;
-	    if (!strncmp (buffer, "From ", i)) {
+	    if (has_prefix(buffer, "From ")) {
 		/* get copy of envelope information ("From " line) */
-		envelope = getcpy (buffer);
+		envelope = mh_xstrdup(buffer);
 
 		/* Put the delivery date in message */
 		fputs (ddate, ffp);
@@ -1275,7 +1277,7 @@ you_lose:
 	return -1;
     }
     fclose (qfp);
-    lseek (fd1, (off_t) 0, SEEK_SET);
+    lseek(fd1, 0, SEEK_SET);
     return fd1;
 
 
@@ -1321,7 +1323,7 @@ trim (char *cp)
 	    *sp = ' ';
 
     /* now return a copy */
-    return getcpy(bp);
+    return mh_xstrdup(bp);
 }
 
 /*
@@ -1334,7 +1336,7 @@ verbose_printf (char *fmt, ...)
     va_list ap;
 
     va_start(ap, fmt);
-    vfprintf (stdout, fmt, ap);
+    vprintf(fmt, ap);
     va_end(ap);
 
     fflush (stdout);	/* now flush output */
@@ -1354,22 +1356,22 @@ adorn (char *what, char *fmt, ...)
     char *s;
 
     eindex = errno;	/* save the errno */
-    fprintf (stdout, ", ");
+    printf(", ");
 
     va_start(ap, fmt);
-    vfprintf (stdout, fmt, ap);
+    vprintf(fmt, ap);
     va_end(ap);
 
     if (what) {
 	if (*what)
-	    fprintf (stdout, " %s: ", what);
+	    printf(" %s: ", what);
 	if ((s = strerror (eindex)))
-	    fprintf (stdout, "%s", s);
+	    fputs(s, stdout);
 	else
-	    fprintf (stdout, "Error %d", eindex);
+	    printf("Error %d", eindex);
     }
 
-    fputc ('\n', stdout);
+    putchar('\n');
     fflush (stdout);
 }
 
@@ -1399,7 +1401,7 @@ static int
 suppress_duplicates (int fd, char *file)
 {
     int	fd1, lockfd, state, result;
-    char *cp, buf[BUFSIZ], name[NAMESZ];
+    char *cp, buf[NMH_BUFSIZ], name[NAMESZ];
     datum key, value;
     DBM *db;
     FILE *in;
@@ -1429,7 +1431,7 @@ suppress_duplicates (int fd, char *file)
 		    continue;
 		}
 
-		cp = add (buf, NULL);
+		cp = mh_xstrdup(buf);
 		while (state == FLDPLUS) {
 		    bufsz = sizeof buf;
 		    state = m_getfld (&gstate, name, buf, &bufsz, in);
