@@ -1,6 +1,4 @@
-
-/*
- * whatnowsbr.c -- the WhatNow shell
+/* whatnowsbr.c -- the WhatNow shell
  *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
@@ -43,6 +41,12 @@
 #include <fcntl.h>
 #include <h/mime.h>
 #include <h/utils.h>
+#ifdef OAUTH_SUPPORT
+# include <h/oauth.h>
+#endif
+#include "../sbr/m_maildir.h"
+#include "../sbr/m_mktemp.h"
+#include "../sbr/mime_type.h"
 
 #define WHATNOW_SWITCHES \
     X("draftfolder +folder", 0, DFOLDSW) \
@@ -53,8 +57,6 @@
     X("prompt string", 4, PRMPTSW) \
     X("version", 0, VERSIONSW) \
     X("help", 0, HELPSW) \
-    X("attach header-field-name", -6, ATTACHSW) \
-    X("noattach", -8, NOATTACHSW) \
 
 
 #define X(sw, minchars, id) id,
@@ -140,7 +142,7 @@ WhatNow (int argc, char **argv)
      *	Get the initial current working directory.
      */
 
-    if (getcwd(cwd, sizeof (cwd)) == (char *)0) {
+    if (getcwd(cwd, sizeof (cwd)) == NULL) {
 	adios("getcwd", "could not get working directory");
     }
 
@@ -193,14 +195,6 @@ WhatNow (int argc, char **argv)
 		if (!(myprompt = *argp++) || *myprompt == '-')
 		    adios (NULL, "missing argument to %s", argp[-2]);
 		continue;
-
-	    case ATTACHSW:
-		advise(NULL, "The -attach switch is deprecated");
-		continue;
-
-	    case NOATTACHSW:
-		advise(NULL, "The -noattach switch is deprecated");
-		continue;
 	    }
 	}
 	if (drft)
@@ -212,7 +206,7 @@ WhatNow (int argc, char **argv)
     if ((drft == NULL && (drft = getenv ("mhdraft")) == NULL) || *drft == 0)
 	drft = getcpy (m_draft (dfolder, dmsg, 1, &isdf));
 
-    msgnam = (cp = getenv ("mhaltmsg")) && *cp ? getcpy (cp) : NULL;
+    msgnam = (cp = getenv ("mhaltmsg")) && *cp ? mh_xstrdup(cp) : NULL;
 
     if ((cp = getenv ("mhatfile")) && *cp)
     	atfile = atoi(cp);
@@ -233,9 +227,9 @@ WhatNow (int argc, char **argv)
     snprintf (prompt, sizeof(prompt), myprompt, invo_name);
     for (;;) {
 #ifdef READLINE_SUPPORT
-	if (!(argp = getans_via_readline (prompt, aleqs))) {
+	if (!(argp = read_switch_multiword_via_readline (prompt, aleqs))) {
 #else /* ! READLINE_SUPPORT */
-	if (!(argp = getans (prompt, aleqs))) {
+	if (!(argp = read_switch_multiword (prompt, aleqs))) {
 #endif /* READLINE_SUPPORT */
 	    (void) m_unlink (LINK);
 	    done (1);
@@ -246,7 +240,7 @@ WhatNow (int argc, char **argv)
 	    if (msgnam)
 		showfile (++argp, msgnam);
 	    else
-		advise (NULL, "no alternate message to display");
+		inform("no alternate message to display");
 	    break;
 
 	case BUILDMIMESW:
@@ -280,7 +274,7 @@ WhatNow (int argc, char **argv)
 		removefile (drft);
 	    } else {
 		if (stat (drft, &st) != NOTOK)
-		    advise (NULL, "draft left on %s", drft);
+		    inform("draft left on %s", drft);
 	    }
 	    done (1);
 
@@ -314,18 +308,17 @@ WhatNow (int argc, char **argv)
 	     *	is accustomed to.  Read back the absolute path.
 	     */
 
-	    if (*(argp+1) == (char *)0) {
-		(void)sprintf(buf, "$SHELL -c \"cd&&pwd\"");
+	    if (*(argp+1) == NULL) {
+		strcpy(buf, "$SHELL -c \"cd&&pwd\"");
 	    }
 	    else {
 		writesomecmd(buf, BUFSIZ, "cd", "pwd", argp);
 	    }
-	    if ((f = popen_in_dir(cwd, buf, "r")) != (FILE *)0) {
-		fgets(cwd, sizeof (cwd), f);
-
-		if (strchr(cwd, '\n') != (char *)0)
-			*strchr(cwd, '\n') = '\0';
-
+	    if ((f = popen_in_dir(cwd, buf, "r")) != NULL) {
+		if (fgets(cwd, sizeof (cwd), f) == NULL) {
+		    advise (buf, "fgets");
+		}
+                trim_suffix_c(cwd, '\n');
 		pclose(f);
 	    }
 	    else {
@@ -336,7 +329,7 @@ WhatNow (int argc, char **argv)
 
 	case PWDCMDSW:
 	    /* Print the working directory for attachments */
-	    printf("%s\n", cwd);
+	    puts(cwd);
 	    break;
 
 	case LSCMDSW:
@@ -360,10 +353,10 @@ WhatNow (int argc, char **argv)
 	    if (checkmimeheader(drft))
 		break;
 
-	    l = (char *)0;
+	    l = NULL;
 	    n = 0;
 
-	    while (*++argp != (char *)0) {
+	    while (*++argp != NULL) {
 		if (strcmp(*argp, "-l") == 0)
 		    l = "/";
 
@@ -382,7 +375,7 @@ WhatNow (int argc, char **argv)
 	    }
 
 	    if (n == -1)
-		advise((char *)0, "usage is alist [-ln].");
+		inform("usage is alist [-ln].");
 
 	    else
 		annolist(drft, ATTACH_FIELD, l, n);
@@ -409,8 +402,8 @@ WhatNow (int argc, char **argv)
 		}
 	    }
 
-	    if (*(argp+1) == (char *)0) {
-		advise(NULL, "attach command requires file argument(s).");
+	    if (*(argp+1) == NULL) {
+		inform("attach command requires file argument(s).");
 		break;
 	    }
 
@@ -428,11 +421,11 @@ WhatNow (int argc, char **argv)
 	     *	draft.
 	     */
 
-	    if ((f = popen_in_dir(cwd, buf, "r")) != (FILE *)0) {
-		while (fgets(shell, sizeof (shell), f) != (char *)0) {
+	    if ((f = popen_in_dir(cwd, buf, "r")) != NULL) {
+		while (fgets(shell, sizeof (shell), f) != NULL) {
 		    char *ctype;
 
-		    *(strchr(shell, '\n')) = '\0';
+                    trim_suffix_c(shell, '\n');
 
 		    if (*shell == '/') {
 		    	strncpy(file, shell, sizeof(file));
@@ -444,9 +437,6 @@ WhatNow (int argc, char **argv)
 		    annotate(drft, ATTACH_FIELD, file, 1, 0, -2, 1);
 		    if (verbose) {
 			ctype = mime_type(file);
-		    }
-
-		    if (verbose) {
 			printf ("Attaching %s as a %s\n", file, ctype);
 			free (ctype);
 		    }
@@ -473,7 +463,7 @@ WhatNow (int argc, char **argv)
 	    if (checkmimeheader(drft))
 		break;
 
-	    for (n = 0, arguments = argp + 1; *arguments != (char *)0; arguments++) {
+	    for (n = 0, arguments = argp + 1; *arguments != NULL; arguments++) {
 		if (strcmp(*arguments, "-n") == 0) {
 			n = 1;
 			break;
@@ -488,15 +478,15 @@ WhatNow (int argc, char **argv)
 	     */
 
 	    if (n == 1) {
-		for (arguments = argp + 1; *arguments != (char *)0; arguments++) {
+		for (arguments = argp + 1; *arguments != NULL; arguments++) {
 		    if (strcmp(*arguments, "-n") == 0)
 			continue;
 
 		    if (**arguments != '\0') {
 			n = atoi(*arguments);
-			annotate(drft, ATTACH_FIELD, (char *)0, 1, 0, n, 1);
+			annotate(drft, ATTACH_FIELD, NULL, 1, 0, n, 1);
 
-			for (argp = arguments + 1; *argp != (char *)0; argp++) {
+			for (argp = arguments + 1; *argp != NULL; argp++) {
 			    if (atoi(*argp) > n) {
 				if (atoi(*argp) == 1)
 				    *argp = "";
@@ -518,9 +508,9 @@ WhatNow (int argc, char **argv)
 	     * provide a file name with a space in it.
 	     */
 	    writelscmd(buf, sizeof(buf), "-d --", argp);
-	    if ((f = popen_in_dir(cwd, buf, "r")) != (FILE *)0) {
-		while (fgets(shell, sizeof (shell), f) != (char *)0) {
-		    *(strchr(shell, '\n')) = '\0';
+	    if ((f = popen_in_dir(cwd, buf, "r")) != NULL) {
+		while (fgets(shell, sizeof (shell), f) != NULL) {
+                    trim_suffix_c(shell, '\n');
 		    annotate(drft, ATTACH_FIELD, shell, 1, 0, 0, 1);
 		}
 		pclose(f);
@@ -532,7 +522,7 @@ WhatNow (int argc, char **argv)
 
 	default:
 	    /* Unknown command */
-	    advise (NULL, "say what?");
+	    inform("say what?");
 	    break;
 	}
     }
@@ -562,7 +552,7 @@ writesomecmd(char *buf, int bufsz, char *cmd, char *trailcmd, char **argp)
      */
     int trailln = strlen(trailcmd) + 4;
     if (ln < 0 || ln + trailln > bufsz)
-	adios((char *)0, "arguments too long");
+	adios(NULL, "arguments too long");
 
     cp = buf + ln;
 
@@ -570,7 +560,7 @@ writesomecmd(char *buf, int bufsz, char *cmd, char *trailcmd, char **argp)
 	ln = strlen(*argp);
 	/* +1 for leading space */
 	if (ln + trailln + 1 > bufsz - (cp-buf))
-	    adios((char *)0, "arguments too long");
+	    adios(NULL, "arguments too long");
 	*cp++ = ' ';
 	memcpy(cp, *argp, ln+1);
 	cp += ln;
@@ -691,7 +681,9 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 	if (atfile) {
 	    (void) m_unlink (linkpath);
 	    if (link (altpath, linkpath) == NOTOK) {
-		symlink (altpath, linkpath);
+		if (symlink (altpath, linkpath) < 0) {
+		    adios (linkpath, "symlink");
+		}
 		slinked = 1;
 	    } else {
 		slinked = 0;
@@ -709,12 +701,15 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 	    break;
 
 	case OK:
-	    if (cwd)
-		chdir (cwd);
+	    if (cwd) {
+		if (chdir (cwd) < 0) {
+		    advise (cwd, "chdir");
+		}
+	    }
 	    if (altmsg) {
 		if (mp)
-		    m_putenv ("mhfolder", mp->foldpath);
-		m_putenv ("editalt", altpath);
+		    setenv("mhfolder", mp->foldpath, 1);
+		setenv("editalt", altpath, 1);
 	    }
 
 	    vec = argsplit(*ed, &prog, &vecp);
@@ -736,9 +731,9 @@ editfile (char **ed, char **arg, char *file, int use, struct msgs *mp,
 		    && (!reedit || (status & 0x00ff))) {
 		    if (!use && (status & 0xff00) &&
 			    (rename (file, cp = m_backup (file)) != NOTOK)) {
-			advise (NULL, "problems with edit--draft left in %s", cp);
+			inform("problems with edit--draft left in %s", cp);
 		    } else {
-			advise (NULL, "problems with edit--%s preserved", file);
+			inform("problems with edit--%s preserved", file);
 		    }
 		}
 		status = -2;	/* maybe "reedit ? -2 : -1"? */
@@ -828,7 +823,8 @@ sendfile (char **arg, char *file, int pushsw)
 	sleep (5);
     switch (child_id) {
 	case NOTOK:
-	    advise (NULL, "unable to fork, so sending directly...");
+	    inform("unable to fork, so sending directly...");
+	    /* FALLTHRU */
 	case OK:
 	    vec = argsplit(sendproc, &program, &vecp);
 	    if (pushsw)
@@ -934,21 +930,21 @@ buildfile (char **argp, char *file)
     X("client host", -6, CLIESW) \
     X("server host", 6, SERVSW) \
     X("snoop", -5, SNOOPSW) \
-    X("draftfolder +folder", -6, SDRFSW) \
-    X("draftmessage msg", -6, SDRMSW) \
-    X("nodraftfolder", -3, SNDRFSW) \
-    X("sasl", SASLminc(-4), SASLSW) \
-    X("nosasl", SASLminc(-6), NOSASLSW) \
-    X("saslmaxssf", SASLminc(-10), SASLMXSSFSW) \
-    X("saslmech", SASLminc(-5), SASLMECHSW) \
-    X("user", SASLminc(-4), USERSW) \
-    X("attach fieldname", 6, SNDATTACHSW) \
-    X("noattach", 0, SNDNOATTACHSW) \
-    X("attachformat", 7, SNDATTACHFORMAT) \
+    X("draftfolder +folder", 0, SDRFSW) \
+    X("draftmessage msg", 0, SDRMSW) \
+    X("nodraftfolder", 0, SNDRFSW) \
+    X("sasl", SASLminc(4), SASLSW) \
+    X("nosasl", SASLminc(6), NOSASLSW) \
+    X("saslmech", SASLminc(5), SASLMECHSW) \
+    X("authservice", SASLminc(0), AUTHSERVICESW) \
+    X("user username", SASLminc(4), USERSW) \
     X("port server-port-name/number", 4, PORTSW) \
     X("tls", TLSminc(-3), TLSSW) \
     X("initialtls", TLSminc(-10), INITTLSSW) \
     X("notls", TLSminc(-5), NTLSSW) \
+    X("certverify", TLSminc(-10), CERTVERSW) \
+    X("nocertverify", TLSminc(-12), NOCERTVERSW) \
+    X("sendmail program", 0, MTSSM) \
     X("mts smtp|sendmail/smtp|sendmail/pipe", 2, MTSSW) \
     X("messageid localname|random", 2, MESSAGEIDSW) \
 
@@ -980,6 +976,9 @@ sendit (char *sp, char **arg, char *file, int pushed)
     int	vecp, n = 1;
     char *cp, buf[BUFSIZ], **argp, *program;
     char **arguments, *savearg[MAXARGS], **vec;
+    const char *user = NULL, *saslmech = NULL;
+    char *auth_svc = NULL;
+    int snoop = 0;
     struct stat st;
 
 #ifndef	lint
@@ -1060,7 +1059,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
 		    ambigsw (cp, sendswitches);
 		    return;
 		case UNKWNSW:
-		    advise (NULL, "-%s unknown\n", cp);
+		    inform("-%s unknown\n", cp);
 		    return;
 
 		case SHELPSW:
@@ -1080,7 +1079,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
 
 		case SPLITSW:
 		    if (!(cp = *argp++) || sscanf (cp, "%d", &splitsw) != 1) {
-			advise (NULL, "missing argument to %s", argp[-2]);
+			inform("missing argument to %s", argp[-2]);
 			return;
 		    }
 		    continue;
@@ -1108,7 +1107,8 @@ sendit (char *sp, char **arg, char *file, int pushed)
 		    continue;
 
 		case DEBUGSW:
-		    debugsw++;	/* fall */
+		    debugsw++;
+		    /* FALLTHRU */
 		case NFILTSW:
 		case FRMTSW:
 		case NFRMTSW:
@@ -1120,56 +1120,66 @@ sendit (char *sp, char **arg, char *file, int pushed)
 		case NMSGDSW:
 		case WATCSW:
 		case NWATCSW:
-		case SNOOPSW:
 		case SASLSW:
 		case NOSASLSW:
 		case TLSSW:
 		case INITTLSSW:
 		case NTLSSW:
+		case CERTVERSW:
+		case NOCERTVERSW:
 		    vec[vecp++] = --cp;
 		    continue;
 
+		case SNOOPSW:
+                    snoop++;
+		    vec[vecp++] = --cp;
+		    continue;
+
+		case AUTHSERVICESW:
+#ifdef OAUTH_SUPPORT
+		    if (!(auth_svc = *argp++) || *auth_svc == '-')
+			adios (NULL, "missing argument to %s", argp[-2]);
+#else
+                    NMH_UNUSED (user);
+                    NMH_UNUSED (auth_svc);
+		    adios (NULL, "not built with OAuth support");
+#endif
+		    continue;
+
+		case SASLMECHSW:
+                    saslmech = *argp;
+		    /* FALLTHRU */
 		case ALIASW:
 		case FILTSW:
 		case WIDTHSW:
 		case CLIESW:
 		case SERVSW:
-		case SASLMXSSFSW:
-		case SASLMECHSW:
 		case USERSW:
 		case PORTSW:
+		case MTSSM:
 		case MTSSW:
 		case MESSAGEIDSW:
 		    vec[vecp++] = --cp;
 		    if (!(cp = *argp++) || *cp == '-') {
-			advise (NULL, "missing argument to %s", argp[-2]);
+			inform("missing argument to %s", argp[-2]);
 			return;
 		    }
 		    vec[vecp++] = cp;
+                    user = cp;
 		    continue;
 
 		case SDRFSW:
 		case SDRMSW:
 		    if (!(cp = *argp++) || *cp == '-') {
-			advise (NULL, "missing argument to %s", argp[-2]);
+			inform("missing argument to %s", argp[-2]);
 			return;
 		    }
+		    continue;
 		case SNDRFSW:
 		    continue;
-
-		case SNDATTACHSW:
-		    advise(NULL, "The -attach switch is deprecated");
-		    continue;
-		case SNDNOATTACHSW:
-		    advise(NULL, "The -noattach switch is deprecated");
-		    continue;
-
-		case SNDATTACHFORMAT:
-		    advise(NULL, "The -attachformat switch is deprecated");
-		    continue;
 	    }
 	}
-	advise (NULL, "usage: %s [switches]", sp);
+	inform("usage: %s [switches]", sp);
 	return;
     }
 
@@ -1177,7 +1187,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
     if ((cp = context_find ("Aliasfile"))) {
 	char **ap, *dp;
 
-	dp = getcpy (cp);
+	dp = mh_xstrdup(cp);
 	for (ap = brkstring (dp, " ", "\n"); ap && *ap; ap++) {
 	    vec[vecp++] = "-alias";
 	    vec[vecp++] = *ap;
@@ -1186,7 +1196,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
 
     if ((cp = getenv ("SIGNATURE")) == NULL || *cp == 0)
 	if ((cp = context_find ("signature")) && *cp)
-	    m_putenv ("SIGNATURE", cp);
+	    setenv("SIGNATURE", cp, 1);
 
     if ((annotext = getenv ("mhannotate")) == NULL || *annotext == 0)
 	annotext = NULL;
@@ -1206,7 +1216,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
 	    adios(NULL, "unable to create temporary file in %s",
 		  get_temp_dir());
 	}
-	distfile = getcpy (cp);
+	distfile = mh_xstrdup(cp);
 	(void) m_unlink(distfile);
 	if (link (altmsg, distfile) == NOTOK)
 	    adios (distfile, "unable to link %s to", altmsg);
@@ -1214,6 +1224,20 @@ sendit (char *sp, char **arg, char *file, int pushed)
 	distfile = NULL;
     }
 
+#ifdef OAUTH_SUPPORT
+    if (auth_svc == NULL) {
+        if (saslmech  &&  ! strcasecmp(saslmech, "xoauth2")) {
+            adios (NULL, "must specify -authservice with -saslmech xoauth2");
+        }
+    } else {
+        if (user == NULL) {
+            adios (NULL, "must specify -user with -saslmech xoauth2");
+        }
+    }
+#else
+    NMH_UNUSED(saslmech);
+#endif /* OAUTH_SUPPORT */
+
     if (altmsg == NULL || stat (altmsg, &st) == NOTOK) {
 	st.st_mtime = 0;
 	st.st_dev = 0;
@@ -1224,7 +1248,7 @@ sendit (char *sp, char **arg, char *file, int pushed)
 
     closefds (3);
 
-    if (sendsbr (vec, vecp, program, file, &st, 1) == OK)
+    if (sendsbr (vec, vecp, program, file, &st, 1, auth_svc) == OK)
 	done (0);
 }
 
@@ -1249,10 +1273,10 @@ whomfile (char **arg, char *file)
 
 	case OK:
 	    vec = argsplit(whomproc, &program, &vecp);
-	    vec[vecp++] = file;
 	    if (arg)
 		while (*arg)
 		    vec[vecp++] = *arg++;
+	    vec[vecp++] = file;
 	    vec[vecp] = NULL;
 
 	    execvp (program, vec);
@@ -1281,7 +1305,7 @@ removefile (char *drft)
 
 
 /*
- * Return 1 if we already have a MIME-Verson header, 0 otherwise.
+ * Return 1 if we already have a MIME-Version header, 0 otherwise.
  */
 
 static int
@@ -1289,7 +1313,7 @@ checkmimeheader (char *drft)
 {
     FILE *f;
     m_getfld_state_t gstate = 0;
-    char buf[BUFSIZ], name[NAMESZ];
+    char buf[NMH_BUFSIZ], name[NAMESZ];
     int state, retval = 0;
 
     if ((f = fopen(drft, "r")) == NULL) {
@@ -1303,7 +1327,7 @@ checkmimeheader (char *drft)
 	case FLD:
 	case FLDPLUS:
 	    if (strcasecmp(name, VRSN_FIELD) == 0) {
-		advise(NULL, "Cannot use attach commands with already-"
+		inform("Cannot use attach commands with already-"
 		       "formatted MIME message \"%s\"", drft);
 		retval = 1;
 		break;
